//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ BitBoards.cpp
//+
//+ Para la utilizacin de bitboards en el programa
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"
#include "Bitboards.h"

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ CONSTANTES DE ROTACION
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

const int Rotado90[64] = {
    H1,H2,H3,H4,H5,H6,H7,H8,
    G1,G2,G3,G4,G5,G6,G7,G8,
    F1,F2,F3,F4,F5,F6,F7,F8,
    E1,E2,E3,E4,E5,E6,E7,E8,
    D1,D2,D3,D4,D5,D6,D7,D8,
    C1,C2,C3,C4,C5,C6,C7,C8,
    B1,B2,B3,B4,B5,B6,B7,B8,
    A1,A2,A3,A4,A5,A6,A7,A8,
};

const int Rotado90Inv[64] = {
    A8,A7,A6,A5,A4,A3,A2,A1,
    B8,B7,B6,B5,B4,B3,B2,B1,
    C8,C7,C6,C5,C4,C3,C2,C1,
    D8,D7,D6,D5,D4,D3,D2,D1,
    E8,E7,E6,E5,E4,E3,E2,E1,
    F8,F7,F6,F5,F4,F3,F2,F1,
    G8,G7,G6,G5,G4,G3,G2,G1,
    H8,H7,H6,H5,H4,H3,H2,H1,
};

const int Rotado45[64] = {
    A2,B3,C4,D5,E6,F7,G8,H1,
    A3,B4,C5,D6,E7,F8,G1,H2,
    A4,B5,C6,D7,E8,F1,G2,H3,
    A5,B6,C7,D8,E1,F2,G3,H4,
    A6,B7,C8,D1,E2,F3,G4,H5,
    A7,B8,C1,D2,E3,F4,G5,H6,
    A8,B1,C2,D3,E4,F5,G6,H7,
    A1,B2,C3,D4,E5,F6,G7,H8 
};

const int Rotado315[64] = {
    A1,B8,C7,D6,E5,F4,G3,H2,
    A2,B1,C8,D7,E6,F5,G4,H3,
    A3,B2,C1,D8,E7,F6,G5,H4,
    A4,B3,C2,D1,E8,F7,G6,H5,
    A5,B4,C3,D2,E1,F8,G7,H6,
    A6,B5,C4,D3,E2,F1,G8,H7,
    A7,B6,C5,D4,E3,F2,G1,H8,
    A8,B7,C6,D5,E4,F3,G2,H1 
};

const int NumDiagonal45[64] = {
    0, 1, 2, 3, 4, 5, 6, 7,
    1, 2, 3, 4, 5, 6, 7, 8,
    2, 3, 4, 5, 6, 7, 8, 9,
    3, 4, 5, 6, 7, 8, 9,10,
    4, 5, 6, 7, 8, 9,10,11,
    5, 6, 7, 8, 9,10,11,12,
    6, 7, 8, 9,10,11,12,13,
    7, 8, 9,10,11,12,13,14,
};

const int NumDiagonal315[64] = {
    7, 6, 5, 4, 3, 2, 1, 0,
    8, 7, 6, 5, 4, 3, 2, 1,
    9, 8, 7, 6, 5, 4, 3, 2,
   10, 9, 8, 7, 6, 5, 4, 3,
   11,10, 9, 8, 7, 6, 5, 4,
   12,11,10, 9, 8, 7, 6, 5,
   13,12,11,10, 9, 8, 7, 6,
   14,13,12,11,10, 9, 8, 7,
};

const int Bitboards::Shift45[64] = {
     9,17,25,33,41,49,57, 1, 
    17,25,33,41,49,57, 1,10,
    25,33,41,49,57, 1,10,19,
    33,41,49,57, 1,10,19,28,
    41,49,57, 1,10,19,28,37,
    49,57, 1,10,19,28,37,46,
    57, 1,10,19,28,37,46,55,
     1,10,19,28,37,46,55,64
};

const int Bitboards::Shift315[64] = {
     1,58,51,44,37,30,23,16,
     9, 1,58,51,44,37,30,23,
    17, 9, 1,58,51,44,37,30,
    25,17, 9, 1,58,51,44,37,
    33,25,17, 9, 1,58,51,44,
    41,33,25,17, 9, 1,58,51,
    49,41,33,25,17, 9, 1,58,
    57,49,41,33,25,17, 9, 1
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ CONSTANTES DE MSB, LSB
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

const Bitboard Bitboards::MSBMask1 = static_cast<Bitboard>(65535) << 16;
const Bitboard Bitboards::MSBMask2 = static_cast<Bitboard>(65535) << 32;
unsigned char Bitboards::TablaMSB[65536];
unsigned char Bitboards::TablaLSB[65536];

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ CONSTANTES DE POPCOUNT
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

const int Bitboards::TablaPopCount[256] = {
	0,1,1,2,1,2,2,3,1,2,2,3,2,3,3,4,
	1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,
	1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,
	2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
	1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,
	2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
	2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
	3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
	1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,
	2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
	2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
	3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
	2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
	3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
	3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
	4,5,5,6,5,6,6,7,5,6,6,7,6,7,7,8,
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ DATOS NAMESPACE
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

Bitboard Bitboards::Mask[64];
Bitboard Bitboards::Mask90[64];
Bitboard Bitboards::Mask45[64];
Bitboard Bitboards::Mask315[64];
Bitboard Bitboards::MaskAlPaso[64];
Bitboard Bitboards::MaskAlfil[64];
Bitboard Bitboards::MaskTorre[64];
Bitboard Bitboards::MaskDama[64];
Bitboard Bitboards::MaskInv[64];
Bitboard Bitboards::MaskInv45[64];
Bitboard Bitboards::MaskInv315[64];
Bitboard Bitboards::MaskInv90[64];

Bitboard Bitboards::MaskFila[8];
Bitboard Bitboards::MaskCol[8];
Bitboard Bitboards::MaskEntre[64][64];
Direccion Bitboards::Direcciones[64][64];

Bitboard Bitboards::MaskNorte[64];
Bitboard Bitboards::MaskSur[64];
Bitboard Bitboards::MaskEste[64];
Bitboard Bitboards::MaskOeste[64];
Bitboard Bitboards::MaskNoreste[64];
Bitboard Bitboards::MaskNoroeste[64];
Bitboard Bitboards::MaskSudeste[64];
Bitboard Bitboards::MaskSudoeste[64];

Bitboard Bitboards::AtaquesCaballos[64];
Bitboard Bitboards::AtaquesReyes[64];
Bitboard Bitboards::AtaquesPeonesB[64];
Bitboard Bitboards::AtaquesPeonesN[64];
Bitboard Bitboards::AtaquesFila[64][64];
Bitboard Bitboards::AtaquesColumna[64][64];
Bitboard Bitboards::AtaquesDiag45[64][64];
Bitboard Bitboards::AtaquesDiag315[64][64];

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ FUNCIONES DE AYUDA
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Encuentra los ataques de una pieza deslizante en una fila,
// conociendo el estado de esa fila y la posicin de la pieza atacante en la fila
int EncontrarAtaques(int casilla, int piezas, int longitud) {
	int ret = 0;

    // Vamos hacia un lado hasta que encontramos una pieza
	if(casilla < 7) {
		int inicio = 1 << (casilla + 1);
		while(inicio < 256) {
			ret |= inicio;
			if(piezas & inicio) break;
			inicio <<= 1;
		}
	}
    // Vamos hacia el otro lado hasta que encontramos una pieza
	if(casilla > 0) {
		int inicio = 1 << (casilla - 1);
		while(inicio > 0) {
			ret |= inicio;
			if(piezas & inicio) break;
			inicio >>= 1;
		}
	}

	return (ret & ((1 << longitud) - 1)); // Se enmascara solo la longitud requerida
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ namespace Bitboards
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Rota un Bitboard 90 a la izquierda
Bitboard Bitboards::RotarBitboard90(Bitboard b) {
	Bitboard rotado = 0;
	for (int i = 0; i < 64; i++) {
        if (b & Mask[i])
            rotado |= Mask90[i];
	}
	return rotado;  
}

// Rota un Bitboard 45 a la izquierda
Bitboard Bitboards::RotarBitboard45(Bitboard b) {
    Bitboard rotado = 0;
	for (int i = 0; i < 64; i++) {
        if (b & Mask[i])
            rotado |= Mask45[i];
	}
    return rotado;
}

// Rota un Bitboard 315 a la izquierda
Bitboard Bitboards::RotarBitboard315(Bitboard b) {
    Bitboard rotado = 0;
	for (int i = 0; i < 64; i++) {
        if (b & Mask[i])
            rotado |= Mask315[i];
	}
    return rotado;
}

// Inicializa los bitboards necesarios para el funcionamiento del programa
void Bitboards::InicializarBitBoards(void) {
    // MSB, LSB
	for (int i = 0; i < 65536; i++) {
		int j;
		for (j = 0; j < 16; j++)
			if (i & (1<<j)) { TablaMSB[i] = (unsigned char)j; break; }
		for (j = 15; j >= 0; j--)
			if (i & (1<<j)) { TablaLSB[i] = (unsigned char)j; break; }
	}

    // Filas, Columnas a cero
    for(int i = 0; i < 8; i++) {
        MaskCol[i] = MaskFila[i] = 0;
    }

    // Mask, Filas, Columnas
    for(Casilla i = A1; i <= H8; i++) {
        Mask[i] = static_cast<Bitboard>(1) << i;
        MaskFila[Fila(i)] |= Mask[i];
        MaskCol[Columna(i)] |= Mask[i];
    }

    // Direccion entre dos casillas
    for(Casilla ini = A1; ini <= H8; ini++) {
        for(Casilla fin = A1; fin <= H8; fin++) {
            if(ini == fin)
                Direcciones[ini][fin] = SIN_DIRECCION;
            else if(Fila(ini) == Fila(fin))
                Direcciones[ini][fin] = (ini < fin) ? ESTE : OESTE;
            else if(Columna(ini) == Columna(fin))
                Direcciones[ini][fin] = (ini < fin) ? NORTE : SUR;
            else if(NumDiagonal315[ini] == NumDiagonal315[fin])
                Direcciones[ini][fin] = (ini < fin) ? NORESTE : SUDOESTE;
            else if(NumDiagonal45[ini] == NumDiagonal45[fin])
                Direcciones[ini][fin] = (ini < fin) ? NOROESTE : SUDESTE;
            else
                Direcciones[ini][fin] = SIN_DIRECCION;
        }
    }
    
    // Mascara de las casillas en medio de dos casillas
    for(int i = 0; i < 64; i++) {
        for(int j = 0; j < 64; j++) {
            Bitboard m = Mask[i], temp = 0; int k = i;
            switch(Direcciones[i][j]) {
                case NORTE: while (k != j) { m <<= 8; temp |= m; k += 8; } temp ^= m; break;
                case SUR: while (k != j) { m >>= 8; temp |= m; k -= 8; } temp ^= m; break;
                case ESTE: while (k != j) { m <<= 1; temp |= m; k += 1; } temp ^= m; break;
                case OESTE: while (k != j) { m >>= 1; temp |= m; k -= 1; } temp ^= m; break;
                case NORESTE: while (k != j) { m <<= 9; temp |= m; k += 9; } temp ^= m; break;
                case SUDOESTE: while (k != j) { m >>= 9; temp |= m; k -= 9; } temp ^= m; break;
                case NOROESTE: while (k != j) { m <<= 7; temp |= m; k += 7; } temp ^= m; break;
                case SUDESTE: while (k != j) { m >>= 7; temp |= m; k -= 7; } temp ^= m; break;
            }
            MaskEntre[i][j] = temp;
        }
	}

    // Mascara de rayos: para cada direccion las casillas que se encuentran en una direccion dada desde una casilla
    for(int origen = 0; origen < 64; origen++) {
        MaskNorte[origen] = MaskSur[origen] = MaskEste[origen] = MaskOeste[origen] = 0;
        MaskNoreste[origen] = MaskNoroeste[origen] = MaskSudeste[origen] = MaskSudoeste[origen] = 0;

        for(int destino = 0; destino < 64; destino++) {
            switch(Direcciones[origen][destino]) {
                case NORTE: AgregarBit(MaskNorte[origen], destino); break;
                case SUR: AgregarBit(MaskSur[origen], destino); break;
                case ESTE: AgregarBit(MaskEste[origen], destino); break;
                case OESTE: AgregarBit(MaskOeste[origen], destino); break;
                case NORESTE: AgregarBit(MaskNoreste[origen], destino); break;
                case NOROESTE: AgregarBit(MaskNoroeste[origen], destino); break;
                case SUDESTE: AgregarBit(MaskSudeste[origen], destino); break;
                case SUDOESTE: AgregarBit(MaskSudoeste[origen], destino); break;
            }
        }
    }

    // Mascaras rotadas
    for(int i = 0; i < 64; i++) {
        Mask90[i] = Mask[Rotado90[i]];
        Mask45[i] = Mask[Rotado45[i]];
        Mask315[i] = Mask[Rotado315[i]];
    }
    
    // Mscaras inversas
    for(int i = 0; i < 64; i++) {
        MaskInv[i] = ~Mask[i];
        MaskInv45[i] = ~Mask45[i];
        MaskInv315[i] = ~Mask315[i];
        MaskInv90[i] = ~Mask90[i];
    }

    // Para la deteccin de si es posible una captura al paso
    for(Casilla i = A1; i <= H8; i++) {
        MaskAlPaso[i] = 0;
        // Capturas al paso para las negras
        if(Fila(i) == Fila4) {
            if(Columna(i) < ColH)
                AgregarBit(MaskAlPaso[i],ColumnaDerecha(i));
            if(Columna(i) > ColA)
                AgregarBit(MaskAlPaso[i],ColumnaIzquierda(i));
        }
        else if(Fila(i) == Fila5) {
            if(Columna(i) < ColH)
                AgregarBit(MaskAlPaso[i],ColumnaDerecha(i));
            if(Columna(i) > ColA)
                AgregarBit(MaskAlPaso[i],ColumnaIzquierda(i));
        }
    }

    // Ataques Rey y Caballo
    int deltaRey[8] = {-9, -7, -8, 7, 9, 8, -1, 1};
	int	deltaCaballo[8] = {-17, -15, -6, 10, 17, 15, 6, -10};
    for(Casilla casilla = A1; casilla <= H8; casilla++) {
        AtaquesReyes[casilla] = 0;
        AtaquesCaballos[casilla] = 0;
        for(int j = 0; j < 8; j++) {
            Casilla pos = casilla + deltaRey[j];
            if(CasillaValida(pos) && Distancia(pos,casilla) == 1)
                AtaquesReyes[casilla] |= Mask[pos];
            pos = casilla + deltaCaballo[j];
            if(CasillaValida(pos) && Distancia(pos,casilla) < 3)
                AtaquesCaballos[casilla] |= Mask[pos];
        }
    }

    // Ataques Peones
    for(Casilla casilla = A1; casilla <= H8; casilla++) {
        AtaquesPeonesB[casilla] = AtaquesPeonesN[casilla] = 0;
        if(Columna(casilla) > ColA) {
            if(Fila(casilla) < Fila8) AtaquesPeonesB[casilla] |= Mask[casilla+7];
            if(Fila(casilla) > Fila1) AtaquesPeonesN[casilla] |= Mask[casilla-9];
        }
        if(Columna(casilla) < ColH) {
            if(Fila(casilla) < Fila8) AtaquesPeonesB[casilla] |= Mask[casilla+9];
            if(Fila(casilla) > Fila1) AtaquesPeonesN[casilla] |= Mask[casilla-7];
        }
    }

    // Ataques Filas
    for(Casilla casilla = A1; casilla <= H8; casilla++) {
        for(int b = 0; b < 64; b++) {
            AtaquesFila[casilla][b] = 0;
            Bitboard ataques = EncontrarAtaques(Columna(casilla), b << 1, 8);
            while(ataques) {
                Casilla cs = MSB(ataques);
                AtaquesFila[casilla][b] |= Mask[(casilla & 56)+cs];
                SacarBit(ataques,cs);
            }
        }
    }

    // Ataques Columnas
    for(Casilla casilla = A1; casilla <= H8; casilla++) {
        for(int b = 0; b < 64; b++) {
            AtaquesColumna[casilla][b] = 0;
            Bitboard ataques = EncontrarAtaques(7-Fila(casilla), b << 1, 8);
            while(ataques) {
                Casilla cs = MSB(ataques);
                AtaquesColumna[casilla][b] |= Mask[Rotado90Inv[((Columna(casilla))<<3)+cs]];
                SacarBit(ataques,cs);
            }
        }
    }

    // Ataques Diagonales: a lo crbmg.cpp
    // TODO: implementar EncontrarAtaques ac
	for (Casilla casilla = A1; casilla <= H8; casilla++) {
		for (int b = 0; b < 64; b++) { 
			Bitboard mask = 0;
			Casilla q = casilla;
			while (Columna(q)>0 && Fila(q)<7) {
				q += 7;
				mask |= Mask[q];
				if (b&(Mask[Rotado45[q]]>>Shift45[casilla]))
					break;
			}
			q = casilla;
			while (Columna(q)<7 && Fila(q)>0) {
				q -= 7;
				mask |= Mask[q];
				if (b&(Mask[Rotado45[q]]>>Shift45[casilla]))
					break;
			}
			AtaquesDiag45[casilla][b] = mask;

			mask = 0;
			q = casilla;
			while (Columna(q)<7 && Fila(q)<7) {
				q += 9;
				mask |= Mask[q];
				if (b&(Mask[Rotado315[q]]>>Shift315[casilla]))
					break;
			}
			q = casilla;
			while (Columna(q)>0 && Fila(q)>0) {
				q -= 9;
				mask |= Mask[q];
				if (b&(Mask[Rotado315[q]]>>Shift315[casilla]))
					break;
			}
			AtaquesDiag315[casilla][b] = mask;
		}
	}

    // Para mejorar la deteccin de amenazas en piezas deslizantes
    for(int i = 0; i < 64; i++) {
		MaskTorre[i] = AtaquesFila[i][0] | AtaquesColumna[i][0];
        MaskAlfil[i] = AtaquesDiag45[i][0] | AtaquesDiag315[i][0];
        MaskDama[i] = MaskTorre[i] | MaskAlfil[i];
    }
}

// Muestra un bitboard en pantalla
void Bitboards::MostrarBitboard(Bitboard b) {
    for(int fila = Fila8; fila >= Fila1; fila--) {
        for(int col = ColA; col <= ColH; col++)
            cout << ((b & Mask[Coord(fila,col)]) ? "x" : ".");
        cout << endl;
    }
	cout << endl << endl;
}
