//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ BitBoards.h
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#pragma once

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ DEFINICIN DE TIPOS
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

typedef EnteroU64 Bitboard;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ CONSTANTES ENUMERADAS
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

enum Direccion {
    SIN_DIRECCION, NORTE, SUR, ESTE, OESTE, NORESTE, NOROESTE, SUDESTE, SUDOESTE
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ namespace Bitboards
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

namespace Bitboards {
    
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ DATOS
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    extern Bitboard Mask[64];
    extern Bitboard Mask90[64];
    extern Bitboard Mask45[64];
    extern Bitboard Mask315[64];
    extern Bitboard MaskAlPaso[64];
    extern Bitboard MaskAlfil[64];
    extern Bitboard MaskTorre[64];
    extern Bitboard MaskDama[64];
    extern Bitboard MaskInv[64];
    extern Bitboard MaskInv45[64];
    extern Bitboard MaskInv315[64];
    extern Bitboard MaskInv90[64];

    extern Bitboard MaskFila[8];
    extern Bitboard MaskCol[8];
    extern Bitboard MaskEntre[64][64];
    extern Direccion Direcciones[64][64];
    
    extern Bitboard MaskNorte[64];
    extern Bitboard MaskSur[64];
    extern Bitboard MaskEste[64];
    extern Bitboard MaskOeste[64];
    extern Bitboard MaskNoreste[64];
    extern Bitboard MaskNoroeste[64];
    extern Bitboard MaskSudeste[64];
    extern Bitboard MaskSudoeste[64];
    
    extern Bitboard AtaquesCaballos[64];
    extern Bitboard AtaquesReyes[64];
    extern Bitboard AtaquesPeonesB[64];
    extern Bitboard AtaquesPeonesN[64];
    extern Bitboard AtaquesFila[64][64];
    extern Bitboard AtaquesColumna[64][64];
    extern Bitboard AtaquesDiag45[64][64];
    extern Bitboard AtaquesDiag315[64][64];

    extern const int Shift45[64];
    extern const int Shift315[64];

    extern const Bitboard MSBMask1;
    extern const Bitboard MSBMask2;
    extern unsigned char TablaMSB[65536];
    extern unsigned char TablaLSB[65536];
    extern const int TablaPopCount[256];
    
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ FUNCIONES INLINE OPERACIONES BIT
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    inline void SacarBit(Bitboard &b, int c) { b &= MaskInv[c]; }
    inline void SacarBit45(Bitboard &b, int c) { b &= MaskInv45[c]; }
    inline void SacarBit315(Bitboard &b, int c) { b &= MaskInv315[c]; }
    inline void SacarBit90(Bitboard &b, int c) { b &= MaskInv90[c]; }
    inline void AgregarBit(Bitboard &b, int c) { b |= Mask[c]; }
    inline void AgregarBit45(Bitboard &b, int c) { b |= Mask45[c]; }
    inline void AgregarBit315(Bitboard &b, int c) { b |= Mask315[c]; }
    inline void AgregarBit90(Bitboard &b, int c) { b |= Mask90[c]; }
    inline void XOR(Bitboard &b, const Bitboard &bits) { b ^= bits; }

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ MSB, LSB y POPCOUNT
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    Casilla MSB(const Bitboard &b);
    Casilla LSB(const Bitboard &b);
    int PopCount(const Bitboard &b);

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ FUNCIONES NO INLINE
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    // Inicializa los bitboards necesarios para el funcionamiento del programa
    void InicializarBitBoards(void);

    // Muestra un bitboard en pantalla
    void MostrarBitboard(Bitboard b);
   
    // Rota un Bitboard 90 a la izquierda
    Bitboard RotarBitboard90(Bitboard b);

    // Rota un Bitboard 45 a la izquierda
    Bitboard RotarBitboard45(Bitboard b);

    // Rota un Bitboard 315 a la izquierda
    Bitboard RotarBitboard315(Bitboard b);
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ MSB, LSB y POPCOUNT
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Devuelve el Most Significant Bit de un Bitboard
inline Casilla Bitboards::MSB(const Bitboard &b) {
    if (b&65535) return static_cast<Casilla>(TablaMSB[b&65535]);
    if (b&MSBMask1) return static_cast<Casilla>(TablaMSB[(b>>16)&65535]+16);
    if (b&MSBMask2) return static_cast<Casilla>(TablaMSB[(b>>32)&65535]+32);
    return static_cast<Casilla>(TablaMSB[b>>48]+48);
}

// Devuelve el Least Significant Bit de un Bitboard
inline Casilla Bitboards::LSB(const Bitboard &b) {
    if (b>>48) return static_cast<Casilla>(TablaLSB[b>>48]+48);
    if ((b>>32)&65535) return static_cast<Casilla>(TablaLSB[(b>>32)&65535]+32);
    if ((b>>16)&65535) return static_cast<Casilla>(TablaLSB[(b>>16)&65535]+16);
    return static_cast<Casilla>((TablaLSB[b&65535]));
}

// Devuelve el Population Count de un Bitboard
inline int Bitboards::PopCount(const Bitboard &b) {
    return TablaPopCount[static_cast<unsigned char>(b)]+TablaPopCount[static_cast<unsigned char>(b>>8)]+
	    TablaPopCount[static_cast<unsigned char>(b>>16)]+TablaPopCount[static_cast<unsigned char>(b>>24)]+
	    TablaPopCount[static_cast<unsigned char>(b>>32)]+TablaPopCount[static_cast<unsigned char>(b>>40)]+
	    TablaPopCount[static_cast<unsigned char>(b>>48)]+TablaPopCount[static_cast<unsigned char>(b>>56)];
}
