//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Casilla.h
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#pragma once

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ CONSTANTES ENUMERADAS
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Casillas del tablero
enum Casilla {
    A1, B1, C1, D1, E1, F1, G1, H1,
    A2, B2, C2, D2, E2, F2, G2, H2,
    A3, B3, C3, D3, E3, F3, G3, H3,
    A4, B4, C4, D4, E4, F4, G4, H4,
    A5, B5, C5, D5, E5, F5, G5, H5,
    A6, B6, C6, D6, E6, F6, G6, H6,
    A7, B7, C7, D7, E7, F7, G7, H7,
    A8, B8, C8, D8, E8, F8, G8, H8,
    Invalida,
};

// Filas
enum Filas {
    Fila1, Fila2, Fila3, Fila4, Fila5, Fila6, Fila7, Fila8
};

// Columnas
enum Columnas {
    ColA, ColB, ColC, ColD, ColE, ColF, ColG, ColH
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ OPERADORES
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

inline Casilla operator+(Casilla casilla, int i) { return static_cast<Casilla>(int(casilla)+i); }
inline Casilla operator-(Casilla casilla, int i) { return static_cast<Casilla>(int(casilla)-i); }
inline Casilla operator*(Casilla casilla, int i) { return static_cast<Casilla>(int(casilla)*i); }
inline bool operator<(Casilla casilla, int i) { return (int(casilla)<i); }
inline Casilla operator++(Casilla& casilla) { casilla = casilla + 1; return casilla; }
inline Casilla operator++(Casilla& casilla, int) { Casilla c = casilla; casilla = casilla + 1; return c; }
inline Casilla& operator+=(Casilla& casilla, int i) { casilla = casilla + i; return casilla; }
inline Casilla& operator-=(Casilla& casilla, int i) { casilla = casilla - i; return casilla; }

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ FUNCIONES AUXILIARES
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

inline bool CasillaValida(Casilla c) { return (c >= A1 && c <= H8); }
inline bool CasillaValidaAlPaso(Casilla c) { return (c >= A1 && c <= Invalida); }
inline bool FilaValida(int f) { return (f >= Fila1 && f <= Fila8); }
inline bool ColumnaValida(int c) { return (c >= ColA && c <= ColH); }
inline int Columna(Casilla c) { return (c & 7); }
inline int Fila(Casilla c) { return (c >> 3); }
inline Casilla Coord(int fila, int columna) { return static_cast<Casilla>(fila * 8 + columna); }
inline int Distancia(Casilla c1, Casilla c2) { return max(abs(Fila(c1)-Fila(c2)),abs(Columna(c1)-Columna(c2))); }
inline Casilla FilaArriba(Casilla c) { return (c + 8); }
inline Casilla FilaAbajo(Casilla c) { return (c - 8); }
inline Casilla ColumnaDerecha(Casilla c) { return (c + 1); }
inline Casilla ColumnaIzquierda(Casilla c) { return (c - 1); }
inline Casilla DosFilasArriba(Casilla c) { return (c + 16); }
inline Casilla DosFilasAbajo(Casilla c) { return (c - 16); }
