//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Consola.cpp
//+
//+ Para representar la consola para introducir comandos y mostrar la salida
//+ por pantalla
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"
#include "Consola.h"

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class Consola
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Constructor
Consola::Consola(void) {
    // Se eliminan los buffers
	setvbuf(stdin,NULL,_IONBF,0);
	setvbuf(stdout,NULL,_IONBF,0);
}

// Destructor
Consola::~Consola(void) { }

// Asigna un motor con la consola
void Consola::AsignarMotor(Motor *m) {
    assert(m != NULL);
    motor = m;
}

// Pasa el control a la consola que manejar la entrada y salida desde ahora
void Consola::Iniciar(void) {
    detenerMotor = false;
    cerrarMotor = false;

	// Creacion de los eventos
	hInputProcesada = CreateEvent(NULL,FALSE,FALSE,NULL);
	hNuevaInput	 = CreateEvent(NULL,FALSE,FALSE,NULL);

	// Creacion del thread
	DWORD dwThreadId; 
    HANDLE hThread; 
    hThread = CreateThread(NULL,0,(DWORD(__stdcall*)(LPVOID))FuncionThread,(LPVOID)this,0,&dwThreadId);
	if (hThread == NULL) {
		Salida() << "CreateThread Failed = " << GetLastError() << endl; 
		exit(1);
	}	

	// El bucle principal
	while (!cerrarMotor) {
		EsperarInput();
		ProcesarInput();
	}

    // Cierre del programa
	CloseHandle(hThread);
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ MANEJO DE THREADS
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// La funcin que lleva a cabo el thread
DWORD Consola::FuncionThread(LPVOID lpParam) {
    // Mecanismo para llamar a un miembro de Consola a travs de una funcin esttica
    return ((Consola*)(lpParam))->Proceso();
}

// Llama a la funcin correspondiente
int Consola::Proceso(void) {
	InputThread();
    return 0; 
}

// Se proces la input y se espera que haya nueva disponible
void Consola::InputProcesada(void) {
    SetEvent(hInputProcesada);
}

// Se recibi nueva input y se espera que se procese
void Consola::NuevaInputRecibida(void) {
    SetEvent(hNuevaInput);
}

// Espera hasta que haya nueva input recibida
void Consola::EsperarInput(void) {
    WaitForSingleObject(hNuevaInput, INFINITE);
}

// Espera hasta que se haya procesada la nueva input recibida
void Consola::EsperarProcesoInput(void) {
    WaitForSingleObject(hInputProcesada,INFINITE);
}

// Chequea si hay nueva input recibida y que debe ser procesada
bool Consola::ChequearInput(void) {
	DWORD ret = WaitForSingleObject(hNuevaInput,0);
	if (ret == WAIT_OBJECT_0)
		return true;
	else
		return false;
}

// Funcion del thread: queda esperando nueva input del usuario y la recibe
void Consola::InputThread(void) {
    char linea[MAX_LINEA];
    char *p, *ret; 

	while (!cerrarMotor) {
		comandoRecibido = "";
		while(!cerrarMotor) {
            // Obtenemos el input del usuario
			ret = fgets(linea,MAX_LINEA,stdin);
			if (ret == NULL) {
				comandoRecibido = "quit";
				break;
			}
			
			// Si se introdujo una lnea completa se la procesa
			p = strchr(linea,'\n');
			if(p)
                linea[p-linea] = '\0';
			comandoRecibido += linea;
			 
            // Se recibi input correcta, se espera para el proceso
            if(p)
                break;
		}

        // Se recibi input, se manda la seal y se espera hasta que se la haya procesado
		NuevaInputRecibida();
		EsperarProcesoInput();
    }
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ MANEJO DEL MOTOR
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void Consola::CerrarMotor(void) {
    cerrarMotor = true;
}

void Consola::DetenerMotor(void) {
    detenerMotor = true;
}

Motor* Consola::GetMotor(void) {
    return motor;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ SALIDA POR PANTALLA
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

SalidaDual& Consola::Salida(void) {
    return *salida;
}
