//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Consola.h
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#pragma once

#include "Motor.h"
#include "Salida.h"

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class Consola
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

class Consola {
public:
    // Constructor
    Consola(void);

    // Destructor
    virtual ~Consola(void);

    // Asigna un motor con la consola
    void AsignarMotor(Motor *m);

    // Pasa el control a la consola que manejar la entrada y salida desde ahora
    void Iniciar(void);

    // Chequea si hay nueva input recibida y que debe ser procesada
    bool ChequearInput(void);

    // Procesa la input recibida
    virtual void ProcesarInput(void) = 0;

    // Devuelve acceso a la salida por pantalla
    SalidaDual& Salida(void);

protected:

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ MANEJO DE THREADS
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    // Funciones para el manejo de threads
    void InputProcesada(void);
    void NuevaInputRecibida(void);
    void EsperarInput(void);
    void EsperarProcesoInput(void);

    // Procesa la entrada y deriva a las funciones correspondientes
    void InputThread(void);

    // La funcin que se utiliza con el thread
    static DWORD FuncionThread(LPVOID lpParam);

    // Llama al proceso de la input
    int Proceso(void);

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ MANEJO DEL MOTOR
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    void CerrarMotor(void);
    void DetenerMotor(void);
    Motor* GetMotor(void);

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ CONSTANTES
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    static const int MAX_LINEA = 1024;

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ DATOS
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    // El motor asociado
    Motor *motor;

    // La salida por pantalla
    SalidaDual *salida;

    bool cerrarMotor;
    bool detenerMotor;

    HANDLE hInputProcesada;
    HANDLE hNuevaInput;

    string comandoRecibido;
};
