//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ ControlTiempo.cpp
//+
//+ El control de la asignacin de tiempo por jugada del motor
//+
//+ Implementa control de tiempo tipo torneo, fischer, o segundos por jugada
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"
#include "ControlTiempo.h"

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class ControlTiempo
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Constructor
ControlTiempo::ControlTiempo(void) {
    Limpiar();
}

// Limpia la informacin
void ControlTiempo::Limpiar(void) {
    tiempoDisponible[Blancas] = tiempoDisponible[Negras] = 0;
    incremento[Blancas] = incremento[Negras] = 0;
    tiempoFijo = 0;
    movesToGo = 0;
    infinito = false;
    profundidad = 0;
}

// Devuelve el tiempo para la jugada
TipoTiempo ControlTiempo::Asignar(Color color) const {
    if(tiempoFijo != 0)
        return tiempoFijo;
    else if(movesToGo)
        return tiempoDisponible[color] / (movesToGo + 1);
    else
        return (tiempoDisponible[color] / 60) + incremento[color];
}

// Establece el tiempo disponible para las blancas
void ControlTiempo::SetDispBlancas(TipoTiempo t) {
    tiempoDisponible[Blancas] = t;
}

// Establece el tiempo disponible para las negras
void ControlTiempo::SetDispNegras(TipoTiempo t) {
    tiempoDisponible[Negras] = t;
}

// Establece un control de tiempo fijo
void ControlTiempo::SetTiempoFijo(TipoTiempo t) {
    tiempoFijo = t;
}

// Establece el incremento por jugada para las blancas
void ControlTiempo::SetIncBlancas(TipoTiempo t) {
    incremento[Blancas] = t;
}

// Establece el incremento por jugada para las negras
void ControlTiempo::SetIncNegras(TipoTiempo t) {
    incremento[Negras] = t;
}

// Establece el numero de jugadas para un nuevo control de tiempo
void ControlTiempo::SetMovesToGo(int m) {
    movesToGo = m;
}

// Establece un control de tiempo infinito
void ControlTiempo::SetInfinito(void) {
    infinito = true;
}

// Establece la profundidad mxima de bsqueda
void ControlTiempo::SetProfundidad(int prof) {
    profundidad = prof;
}

// Devuelve true si el control de tiempo es infinito
bool ControlTiempo::EsInfinito(void) const {
    return infinito;
}

// Devuelve la profundidad en un control de tiempo por profundidad
int ControlTiempo::GetProfundidad(void) const {
    return profundidad;
}

