//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Enroque.h
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#pragma once

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class InfoEnroque
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

class InfoEnroque {
public:

    // SETTERS
    void SetDerechosInicial(bool ecb, bool ecn, bool elb, bool eln);
    void SetBlancasPuedenEnrocarCorto(void);
    void SetNegrasPuedenEnrocarCorto(void);
    void SetBlancasPuedenEnrocarLargo(void);
    void SetNegrasPuedenEnrocarLargo(void);
    void DesactivarEnroquesBlancos(void);
    void DesactivarEnroquesNegros(void);
    void DesactivarEnroqueCortoBlanco(void);
    void DesactivarEnroqueCortoNegro(void);
    void DesactivarEnroqueLargoBlanco(void);
    void DesactivarEnroqueLargoNegro(void);
    void SetHechoEnroqueCortoBlanco(void);
    void SetHechoEnroqueLargoBlanco(void);
    void SetHechoEnroqueCortoNegro(void);
    void SetHechoEnroqueLargoNegro(void);

    // GETTERS
    bool BlancasPuedenEnrocarCorto(void) const;
    bool BlancasPuedenEnrocarLargo(void) const;
    bool NegrasPuedenEnrocarCorto(void) const;
    bool NegrasPuedenEnrocarLargo(void) const;
    bool BlancasEnrocaronCorto(void) const;
    bool NegrasEnrocaronCorto(void) const;
    bool BlancasEnrocaronLargo(void) const;
    bool NegrasEnrocaronLargo(void) const;
    int GetDerechosHashkey(void) const;

private:

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ CONSTANTES ENUMERADAS
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    enum Enroques {
        ECB = 1, ECN = 2, ELB = 4, ELN = 8,
        ECBH = 16, ECNH = 32, ELBH = 64, ELNH = 128
    };

    int enroque; // Donde se guarda todo
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ SETTERS
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Establece los derechos de enroque
inline void InfoEnroque::SetDerechosInicial(bool ecb, bool ecn, bool elb, bool eln) {
    enroque = 0;
    if(ecb) SetBlancasPuedenEnrocarCorto();
    if(ecn) SetNegrasPuedenEnrocarCorto();
    if(elb) SetBlancasPuedenEnrocarLargo();
    if(eln) SetNegrasPuedenEnrocarLargo();
}

// Establece que las blancas pueden enrocar corto
inline void InfoEnroque::SetBlancasPuedenEnrocarCorto(void) {
    enroque |= ECB;
}

// Establece que las negras pueden enrocar corto
inline void InfoEnroque::SetNegrasPuedenEnrocarCorto(void) {
    enroque |= ECN;
}

// Establece que las blancas pueden enrocar largo
inline void InfoEnroque::SetBlancasPuedenEnrocarLargo(void) {
    enroque |= ELB;
}

// Establece que las negras pueden enrocar largo
inline void InfoEnroque::SetNegrasPuedenEnrocarLargo(void) {
    enroque |= ELN;
}

// Desactiva un tipo de enroque
inline void InfoEnroque::DesactivarEnroquesBlancos(void) {
    enroque &= ~(ECB | ELB);
}

// Desactiva un tipo de enroque
inline void InfoEnroque::DesactivarEnroquesNegros(void) {
    enroque &= ~(ECN | ELN);
}

// Desactiva el enroque corto blanco
inline void InfoEnroque::DesactivarEnroqueCortoBlanco(void) {
    enroque &= ~ECB;
}

// Desactiva el enroque corto negro
inline void InfoEnroque::DesactivarEnroqueCortoNegro(void) {
    enroque &= ~ECN;
}

// Desactiva el enroque largo blanco
inline void InfoEnroque::DesactivarEnroqueLargoBlanco(void) {
    enroque &= ~ELB;
}

// Desactiva el enroque largo negro
inline void InfoEnroque::DesactivarEnroqueLargoNegro(void) {
    enroque &= ~ELN;
}

// Establece que el enroque corto blanco ha sido realizado
inline void InfoEnroque::SetHechoEnroqueCortoBlanco(void) {
    enroque |= ECBH;
}

// Establece que el enroque largo blanco ha sido realizado
inline void InfoEnroque::SetHechoEnroqueLargoBlanco(void) {
    enroque |= ELBH;
}

// Establece que el enroque corto negro ha sido realizado
inline void InfoEnroque::SetHechoEnroqueCortoNegro(void) {
    enroque |= ECNH;
}

// Establece que el enroque largo negro ha sido realizado
inline void InfoEnroque::SetHechoEnroqueLargoNegro(void) {
    enroque |= ELNH;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ GETTERS
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Devuelve true si las blancas pueden enrocar corto
inline bool InfoEnroque::BlancasPuedenEnrocarCorto(void) const {
    return ((enroque & ECB) != 0);
}

// Devuelve true si las blancas pueden enrocar largo
inline bool InfoEnroque::BlancasPuedenEnrocarLargo(void) const {
    return ((enroque & ELB) != 0);
}    

// Devuelve true si las negras pueden enrocar corto
inline bool InfoEnroque::NegrasPuedenEnrocarCorto(void) const {
    return ((enroque & ECN) != 0);
}

// Devuelve true si las negras pueden enrocar largo
inline bool InfoEnroque::NegrasPuedenEnrocarLargo(void) const {
    return ((enroque & ELN) != 0);
}

// Devuelve true si las blancas enrocaron corto
inline bool InfoEnroque::BlancasEnrocaronCorto(void) const {
    return ((enroque & ECBH) != 0);
}

// Devuelve true si las negras enrocaron corto
inline bool InfoEnroque::NegrasEnrocaronCorto(void) const {
    return ((enroque & ECNH) != 0);
}

// Devuelve true si las blancas enrocaron largo
inline bool InfoEnroque::BlancasEnrocaronLargo(void) const {
    return ((enroque & ELBH) != 0);
}

// Devuelve true si las negras enrocaron largo
inline bool InfoEnroque::NegrasEnrocaronLargo(void) const {
    return ((enroque & ELNH) != 0);
}

// Devuelve los derechos en un entero para las hashkeys
inline int InfoEnroque::GetDerechosHashkey(void) const {
    return (enroque & 15);
}

