//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ EstadoTablero.h
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#pragma once

class EstadoTablero {
public:

    // Constructor
    EstadoTablero();

    void Limpiar(void);

    // LADO QUE MUEVE
    Color GetLadoMueve(void) const;
    bool BlancasMueven(void) const;
    void SetLadoMueve(Color lado);
    void SetBlancasMueven(void);
    void SetNegrasMueven(void);
    void CambiarLado(void);

    // CASILLA AL PASO
    Casilla GetCasillaAlPaso(void) const;
    void SetCasillaAlPaso(Casilla c);

    // ENROQUES
    InfoEnroque& GetInfoEnroque(void);
    InfoEnroque GetInfoEnroque(void) const;
    void SetInfoEnroque(InfoEnroque enroque);

    // REGLA DE LAS CINCUENTA JUGADAS
    int GetCincuenta(void) const;
    bool EsReglaCincuentaAplicable(void) const;
    void SetCincuenta(int cincuenta);
    void IncrementarCincuenta(void);
    void ReiniciarReglaCincuenta(void);

private:

    bool            blancasMueven;
    Casilla         casillaAlPaso;
    InfoEnroque     infoEnroque;
    int             reglaCincuenta;    
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ CONSTRUCTOR
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

inline EstadoTablero::EstadoTablero() {
    Limpiar();
}

// Establece los datos en cero
inline void EstadoTablero::Limpiar(void) {
    blancasMueven = true;
    casillaAlPaso = Invalida;
    reglaCincuenta = 0;
    infoEnroque.SetDerechosInicial(false,false,false,false);
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ LADO QUE MUEVE
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Devuelve true si las blancas mueven
inline bool EstadoTablero::BlancasMueven(void) const {
    return blancasMueven;
}

// Establece el lado que mueve
inline void EstadoTablero::SetLadoMueve(Color lado) {
    assert(ColorValido(lado));
    blancasMueven = (lado == Blancas) ? true : false;
}

// Establece que las blancas mueven
inline void EstadoTablero::SetBlancasMueven(void) {
    blancasMueven = true;
}

// Establece que las negras mueven
inline void EstadoTablero::SetNegrasMueven(void) {
    blancasMueven = false;
}

// Cambia el lado que mueve
inline void EstadoTablero::CambiarLado(void) {
    blancasMueven = (blancasMueven) ? false : true;
}

// Devuelve el lado que mueve
inline Color EstadoTablero::GetLadoMueve(void) const {
    return (blancasMueven ? Blancas : Negras);
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ CASILLA AL PASO
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


// Establece la casilla de captura al paso
inline void EstadoTablero::SetCasillaAlPaso(Casilla c) {
    assert(CasillaValida(c) || c == Invalida);
    casillaAlPaso = c;
}

// Devuelve la casilla de captura al paso
inline Casilla EstadoTablero::GetCasillaAlPaso(void) const {
    return casillaAlPaso;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ REGLA DE LAS CINCUENTA JUGADAS
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Establece el numero de jugadas desde la ultima captura o movimiento de peon
inline void EstadoTablero::SetCincuenta(int cincuenta) {
    assert(cincuenta >= 0 && cincuenta <= 100);
    reglaCincuenta = cincuenta;
}

// Incrementa la regla de las cincuenta
inline void EstadoTablero::IncrementarCincuenta(void) {
    reglaCincuenta++;
}

// Establece la regla de las cincuenta en cero
inline void EstadoTablero::ReiniciarReglaCincuenta(void) {
    reglaCincuenta = 0;
}

// Devuelve el numero de jugadas desde la ultima captura o movimiento de peon
inline int EstadoTablero::GetCincuenta(void) const {
    return reglaCincuenta;
}

// Devuelve true si se pasaron los cincuenta movimientos sin capturas o movimiento de pen
inline bool EstadoTablero::EsReglaCincuentaAplicable(void) const {
    return reglaCincuenta > 100;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ ENROQUES
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Establece la info del enroque
inline void EstadoTablero::SetInfoEnroque(InfoEnroque enroque) {
    infoEnroque = enroque;
}

// Devuelve los derechos de enroque (no const)
inline InfoEnroque& EstadoTablero::GetInfoEnroque(void) {
    return infoEnroque;
}

// Devuelve los derechos de enroque (const)
inline InfoEnroque EstadoTablero::GetInfoEnroque(void) const {
    return infoEnroque;
}
