//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Evaluador.cpp
//+
//+ Para evaluar estticamente una posicin
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"
#include "Evaluador.h"
#include "Tablero.h"
#include "Bitboards.h"

using namespace Bitboards;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ EVALUACION PARTIDA
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

static const int POCO_DESARROLLO_PIEZAS_MENORES = -25;
static const int POCO_DESARROLLO_PEONES = -60;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ TABLAS PIEZA CASILLA
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

static const int ESPEJO[64] = {
    56,57,58,59,60,61,62,63,
    48,49,50,51,52,53,54,55,
    40,41,42,43,44,45,46,47,
    32,33,34,35,36,37,38,39,
    24,25,26,27,28,29,30,31,
    16,17,18,19,20,21,22,23,
     8, 9,10,11,12,13,14,15,
     0, 1, 2, 3, 4, 5, 6, 7
};	

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ EVALUACION REYES
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

static const int T_REY[64] = {
      2, 14,  0,  0,  0,  0, 14,  2,
     -3, -5, -6, -6, -6, -6, -5, -3,
     -5, -5, -8, -8, -8, -8, -5, -5,
     -8, -8,-13,-13,-13,-13, -8, -8,
    -13,-13,-21,-21,-21,-21,-13,-13,
    -21,-21,-34,-34,-34,-34,-21,-21,
    -34,-34,-55,-55,-55,-55,-34,-34,
    -55,-55,-89,-89,-89,-89,-55,-55,
};

static const int ENROQUE_CORTO = 30;
static const int ENROQUE_LARGO = 20;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ EVALUACION PEONES
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

static const int T_PEON[64] = {
     0, 0, 0, 0, 0, 0, 0, 0, // Blancas
     0, 0, 0, 0, 0, 0, 0, 0,
     0, 3, 8,11,11, 8, 3, 0,
     3, 6,11,15,15,11, 6, 3,
     6, 9,14,17,17,14, 9, 6,
     9,12,15,18,18,15,12, 9,
    12,15,18,21,21,18,15,12,
     0, 0, 0, 0, 0, 0, 0, 0, // Negras
};

static const int PEON_AISLADO[8] = { -5,-6,-8,-10,-10,-8,-6,-5 };
static const int PEON_DOBLADO[8] = { -4,-5,-6,-8,-8,-6,-5,-4 };
static const int PEON_ATRASADO[8] = { -2,-3,-4,-10,-10,-4,-3,-2 };

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ EVALUACION TORRES
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

static const int T_TORRE[64] = {
	0,0,4,6,6,4,0,0,
	0,0,4,6,6,4,0,0,
	0,0,4,6,6,4,0,0,
	0,0,4,6,6,4,0,0,
	0,0,4,6,6,4,0,0,
	0,0,4,6,6,4,0,0,
	0,0,4,6,6,4,0,0,
	0,0,4,6,6,4,0,0,
};

static const int TORRE_COLUMNA_SEMIABIERTA[8] = { 3,6,8,12,12,8,6,3 };
static const int TORRE_COLUMNA_ABIERTA[8] = { 5,8,10,15,15,10,8,5 };
static const int TORRE_DOBLADA[8] = { 6,9,12,18,18,12,9,6 };
static const int TORRE_SEPTIMA = 30;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ EVALUACION ALFILES
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

static const int T_ALFIL[64] = {
	-2,-2,-2,-2,-2,-2,-2,-2,
	-2, 8, 5, 5, 5, 5, 8,-2,
	-2, 3, 3, 5, 5, 3, 3,-2,
	-2, 2, 5, 4, 4, 5, 2,-2,
	-2, 2, 5, 4, 4, 5, 2,-2,
	-2, 3, 3, 5, 5, 3, 3,-2,
	-2, 8, 5, 5, 5, 5, 8,-2,
	-2,-2,-2,-2,-2,-2,-2,-2,
};

static const int ALFIL_ATRAPADO = -50;
static const int PAR_ALFILES = 30;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ EVALUACION CABALLOS
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

static const int T_CABALLO[64] = {
	-20,-12,-10,-10,-10,-10,-12,-20,
	-14, -8,  0,  3,  3,  0, -8,-14,
	-13, -2,  5,  5,  5,  5, -2,-13,
	-12, -2,  5, 10, 10,  5, -2,-12,
	-12, -2,  5, 10, 10,  5, -2,-12,
	-13, -2,  5,  5,  5,  5, -2,-13,
	-14, -8,  0,  3,  3,  0, -8,-14,
	-20,-12,-10,-10,-10,-10,-12,-20,
};

static const int CABALLO_ATRAPADO = -40;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ EVALUACION DAMAS
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

static const int T_DAMA[64] = {
	-10,-8,-4, 0, 0,-4,-8,-10,
	-10, 2, 8, 8, 8, 8, 2,-10,
	-10,-6, 4, 8, 8, 4,-6,-10,
	-10, 2, 6,10,10, 6, 2,-10,
	 -2, 2, 6,10,10, 6, 2, -2,
	 -2, 2, 4, 8, 8, 4, 2, -2,
	-12,-2, 4, 5, 5, 4,-2,-12,
	-10,-8,-4, 0, 0,-4,-8,-10,
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class Evaluador
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Constructor
Evaluador::Evaluador(Motor &m) : motor(m) {

}

// Evala el tablero y devuelve su puntaje
int Evaluador::Evaluar(const Tablero &tablero) {
    Bitboard piezas;
    Casilla c;
    
    // El Material
    int puntaje = tablero.DiferenciaMaterial();
    
    // Evaluacin peones blancos
    piezas = tablero.PeonesB();
    while(piezas) {
        c = MSB(piezas);
        SacarBit(piezas, c);

        puntaje += T_PEON[c];
    }

    // Evaluacin peones negros
    piezas = tablero.PeonesN();
    while(piezas) {
        c = MSB(piezas);
        SacarBit(piezas, c);
        
        puntaje -= T_PEON[ESPEJO[c]];
    }

    // Evaluacin caballos blancos
    piezas = tablero.CaballosB();
    while(piezas) {
        c = MSB(piezas);
        SacarBit(piezas, c);

        puntaje += T_CABALLO[c];
    }

    // Evaluacin caballos negros
    piezas = tablero.CaballosN();
    while(piezas) {
        c = MSB(piezas);
        SacarBit(piezas, c);

        puntaje -= T_CABALLO[ESPEJO[c]];
    }

    // Evaluacin alfiles blancos
    piezas = tablero.AlfilesB();

    // Par de alfiles
    if(PopCount(piezas) == 2)
        puntaje += PAR_ALFILES;

    while(piezas) {
        c = MSB(piezas);
        SacarBit(piezas, c);

        puntaje += T_ALFIL[c];
    }

    // Evaluacin alfiles negros
    piezas = tablero.AlfilesN();

    // Par de alfiles
    if(PopCount(piezas) == 2)
        puntaje -= PAR_ALFILES;

    while(piezas) {
        c = MSB(piezas);
        SacarBit(piezas, c);

        puntaje -= T_ALFIL[ESPEJO[c]];
    }

    // Evaluacin torres blancas
    piezas = tablero.TorresB();
    while(piezas) {
        c = MSB(piezas);
        SacarBit(piezas, c);

        // Torre en sptima
        if(Fila(c) == Fila7)
            puntaje += TORRE_SEPTIMA;

        // Torre en columna abierta y semiabierta
        if(!(MaskCol[c] & tablero.PeonesN())) {
            if(!(MaskCol[c] & tablero.PeonesB())) {
                puntaje += TORRE_COLUMNA_ABIERTA[Columna(c)];

                // Torre doblada en columna abierta
                if(MaskCol[c] & piezas)
                    puntaje += TORRE_DOBLADA[Columna(c)];
            }
            else
                puntaje += TORRE_COLUMNA_SEMIABIERTA[Columna(c)];
        }

        puntaje += T_TORRE[c];
    }

    // Evaluacin torres negras
    piezas = tablero.TorresN();
    while(piezas) {
        c = MSB(piezas);
        SacarBit(piezas, c);

        // Torre en sptima
        if(Fila(c) == Fila2)
            puntaje -= TORRE_SEPTIMA;

        // Torre en columna abierta y semiabierta
        if(!(MaskCol[c] & tablero.PeonesB())) {
            if(!(MaskCol[c] & tablero.PeonesN())) {
                puntaje -= TORRE_COLUMNA_ABIERTA[Columna(c)];


                // Torre doblada en columna abierta
                if(MaskCol[c] & piezas)
                    puntaje -= TORRE_DOBLADA[Columna(c)];
            }
            else
                puntaje -= TORRE_COLUMNA_SEMIABIERTA[Columna(c)];
        }

        puntaje -= T_TORRE[ESPEJO[c]];
    }

    // Evaluacin damas blancas
    piezas = tablero.DamasB();
    while(piezas) {
        c = MSB(piezas);
        SacarBit(piezas, c);

        puntaje += T_DAMA[c];
    }

    // Evaluacin damas negras
    piezas = tablero.DamasN();
    while(piezas) {
        c = MSB(piezas);
        SacarBit(piezas, c);

        puntaje -= T_DAMA[ESPEJO[c]];
    }

    // Evaluacin rey blanco
    puntaje += T_REY[tablero.GetPosicionReyB()];
    if(tablero.GetEstado().GetInfoEnroque().BlancasEnrocaronCorto())
        puntaje += ENROQUE_CORTO;
    else if(tablero.GetEstado().GetInfoEnroque().BlancasEnrocaronLargo())
        puntaje += ENROQUE_LARGO;
        
    // Evaluacin rey negro
    puntaje -= T_REY[ESPEJO[tablero.GetPosicionReyN()]];
    if(tablero.GetEstado().GetInfoEnroque().NegrasEnrocaronCorto())
        puntaje -= ENROQUE_CORTO;
    else if(tablero.GetEstado().GetInfoEnroque().NegrasEnrocaronLargo())
        puntaje -= ENROQUE_LARGO;
    
    // Ajustamos segn el color
    return((tablero.GetEstado().BlancasMueven()) ? puntaje : -puntaje);
}
