//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Hashkeys.h
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#pragma once

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ DEFINICIN DE TIPOS
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

typedef EnteroU64 Hashkey;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ namespace Hashkeys
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

namespace Hashkeys {

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ DATOS
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    extern Hashkey HashPiezas[MAX_PIEZAS][64];
    extern Hashkey HashEnroque[16];
    extern Hashkey HashAlPaso[65];
    extern Hashkey HashMovNulo;

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ FUNCIONES NO INLINE
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    void IniciarHashKeys();

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ FUNCIONES INLINE
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    void ActualizarHash(Hashkey &key, Pieza pieza, Casilla casilla);
    void ActualizarHashEnroque(Hashkey &key, InfoEnroque enroque);
    void ActualizarHashAlPaso(Hashkey &key, Casilla ap);
    void ActualizarHashLado(Hashkey &key);
    void ActualizarHashMovimientoNulo(Hashkey &key);
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ FUNCIONES INLINE
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

inline void Hashkeys::ActualizarHash(Hashkey &key, Pieza pieza, Casilla casilla) {
    assert(PiezaValida(pieza));
    assert(CasillaValida(casilla));
    key ^= HashPiezas[int(pieza)][casilla];
}

inline void Hashkeys::ActualizarHashEnroque(Hashkey &key, InfoEnroque enroque) {
    key ^= HashEnroque[enroque.GetDerechosHashkey()];
}

inline void Hashkeys::ActualizarHashAlPaso(Hashkey &key, Casilla ap) {
    assert(CasillaValida(ap) || ap == Invalida);
    key ^= HashAlPaso[ap];
}

inline void Hashkeys::ActualizarHashLado(Hashkey &key) {
    key = ~key;
}

inline void Hashkeys::ActualizarHashMovimientoNulo(Hashkey &key) {
    key ^= HashMovNulo;
}
