//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Historial.h
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#pragma once

#include "Movimiento.h"
#include "Hashkeys.h"

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class Historial
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

class Historial {
public:
    // Constructor
    Historial(void);

    // Muestra el historial por pantalla
    void Mostrar(void) const;

    // Getters
    int GetPly(void) const;
    Movimiento GetMovimiento(void) const;
    Movimiento GetMovimientoAnterior(void) const;
    EstadoTablero GetEstadoTablero(void) const;
    Hashkey GetHashkey(void) const;
    Hashkey GetHashKey(int p) const;

    // Setters
    void Limpiar(void);
    void AumentarPly(void);
    void ReducirPly(void);
    void Agregar(Movimiento m, const Hashkey &hash, const EstadoTablero &e);
    void AgregarNulo(const EstadoTablero &e);

private:
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ struct InfoHistorial
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    struct InfoHistorial {
    public:
        Movimiento movimiento;
        EstadoTablero estado;
        Hashkey hashkey;
    };

    // El mximo nmero de jugadas que podemos guardar
    static const int MAX_HISTORIAL = 1024;

    // El historial propiamente dicho
    InfoHistorial historial[MAX_HISTORIAL];
    
    // La posicin en el historial
    int ply;
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ OPERADORES
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Para mostrar el historial por pantalla
extern ostream& operator<<(ostream& o, const Historial &h);

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ FUNCIONES INLINE
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Limpia el historial
inline void Historial::Limpiar(void) {
    ply = 0;
}

// Obtiene el ply actual
inline int Historial::GetPly(void) const {
    return ply;
}

// Avanza en el historial
inline void Historial::AumentarPly(void) {
    assert(ply < MAX_HISTORIAL);
    ply++;
}

// Retrocede en el historial
inline void Historial::ReducirPly(void) {
    assert(ply > 0);
    ply--;
}

// Obtiene el movimiento actual
inline Movimiento Historial::GetMovimiento(void) const {
    return historial[ply].movimiento;
}

// Obtiene el movimiento anterior al actual
inline Movimiento Historial::GetMovimientoAnterior(void) const {
    assert(ply > 0);
    return historial[ply-1].movimiento;
}

// Obtiene el estado del tablero
inline EstadoTablero Historial::GetEstadoTablero(void) const {
    return historial[ply].estado;
}

// Obtiene la hashkey
inline Hashkey Historial::GetHashkey(void) const {
    return historial[ply].hashkey;
}

// Obtiene la hashkey de un ply especificado (para el conteo de repeticiones)
inline Hashkey Historial::GetHashKey(int p) const {
    assert(p >= 0 && p <= ply);
    return historial[p].hashkey;
}

// Agrega un movimiento al historial
inline void Historial::Agregar(Movimiento m, const Hashkey &hash, const EstadoTablero &e) {
    historial[ply].movimiento = m;
    historial[ply].estado = e;
    historial[ply].hashkey = hash;
}

// Agrega un movimiento nulo al historial
inline void Historial::AgregarNulo(const EstadoTablero &e) {
    historial[ply].estado = e;
}
