//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ ListaMovimientos.h
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#pragma once

#include "Movimiento.h"

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ DECLARACION PREVIA DE CLASES
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

class Arbol;
class Tablero;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ CONSTANTES
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

enum TipoLista {
    PERFT,         // Capturas y No Capturas, sin distincion de fases
    COMPLETA,      // Capturas y No Capturas, con distincion de fases
    COMPLETA_RAIZ, // Capturas y No Capturas, con un ordenado especial para raz
    EVASIONES,     // Evasiones al jaque (debe utilizar GetSiguienteEvasion())
    CAPTURAS,      // Capturas con distincion de fases (debe utilizar GetSiguienteCaptura())
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class ListaMovimientos
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

class ListaMovimientos {
public:
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ CONSTRUCTORES
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    // Constructor
    ListaMovimientos(void);
    ListaMovimientos(const Tablero *tab);
    
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ ESTABLECIMIENTO DEL TIPO DE LISTA
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    // Establece el arbol que contiene a la lista
    void SetArbol(const Arbol *a);

    // Establece el tablero al cual corresponde la lista de movimientos
    void SetTablero(const Tablero *t);

    // Establece el tipo de lista
    void SetTipoLista(TipoLista tipoLista);
    
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ OBTENCION DEL SIGUIENTE MOVIMIENTO
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    
    // Devuelve el siguiente movimiento en la lista
    Movimiento GetSiguiente(int ply = 0);

    // Devuelve el puntaje del ultimo movimiento devuelto
    inline int GetPuntajeUltimoMov(void) const;

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ OTRAS OPERACIONES
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    // Reinicia la lista de movimientos
    void Reiniciar(void);

    int Cantidad(void) const;

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ AGREGADO DE MOVIMIENTOS A LA LISTA
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    void AgregarSimple(Casilla origen, Casilla destino, Pieza pieza);
    void AgregarCaptura(Casilla origen, Casilla destino, Pieza pieza, Pieza captura);
    void AgregarPromocionBlanca(Casilla origen, Casilla destino, Pieza promocion, Pieza captura);
    void AgregarPromocionNegra(Casilla origen, Casilla destino, Pieza promocion, Pieza captura);
    void AgregarAlPasoBlanco(Casilla origen, Casilla destino);
    void AgregarAlPasoNegro(Casilla origen, Casilla destino);
    void AgregarECB(void);
    void AgregarECN(void);
    void AgregarELB(void);
    void AgregarELN(void);

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ CONSTANTES
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    static const int MAX_MOVIMIENTOS = 256; // Maxima cantidad de movimientos guardados

private:
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ OBTENCION DEL SIGUIENTE MOVIMIENTO
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    // Devuelve el siguiente movimiento en la lista (para listas tipos COMPLETA y PERFT)
    Movimiento GetSiguienteCompleta(int ply = 0);

    // Devuelve la siguiente evasin al jaque en la lista (para lista tipo EVASIONES)
    Movimiento GetSiguienteEvasion(int ply = 0);

    // Devuelve la siguiente captura en la lista (para lista tipo CAPTURAS)
    Movimiento GetSiguienteCaptura(int ply = 0);

    // Devuelve el siguiente movimiento en la lista (para lista tipo COMPLETA_RAIZ)
    Movimiento GetSiguienteRaiz(int ply = 0);

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ FUNCIONES
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    // Ordena los movimientos en la lista
    void Ordenar(void);

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ CONSTANTES
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    static const int PUNTAJE_EVITAR_MOVIMIENTO = -99999; // Puntaje que se le asigna a un movimiento para no usarlo

    enum FaseLista {
        PROBAR_HASH,                // Proba el movimiento de la tabla hash
        GENERAR_CAPTURAS,           // Genera las capturas y las ordena
        ORDENAR_CAPTURAS,           // Ordena las capturas
        PROBAR_CAPTURAS_GANADORAS,  // Prueba con la lista de capturas ganadoras
        PROBAR_KILLER1,             // Prueba el killer1
        PROBAR_KILLER2,             // Prueba el killer2
        GENERAR_NOCAPTURAS,         // Genera el resto de los movimientos
        ORDENAR_MOVIMIENTOS,        // Ordena todos los movimientos
        PASAR_RESTO_MOVS,           // Prueba las capturas perdedoras y el resto de los movimientos
        FIN_MOVS,                   // Se acabaron los movimiento, corta los bucles
    };

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ DATOS
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    const Tablero *tablero;
    const Arbol *arbol;
    
    Movimiento lista[MAX_MOVIMIENTOS];
    int puntajes[MAX_MOVIMIENTOS];
  
    int movActual;
    int cantidad;
    int puntajeActual;
    
    TipoLista tipo;
    FaseLista siguienteFase;
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ FUNCIONES INLINE
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Constructor
inline ListaMovimientos::ListaMovimientos(void) : arbol(NULL), tablero(NULL) {}
inline ListaMovimientos::ListaMovimientos(const Tablero *tab) : arbol(NULL), tablero(tab) {}

// Devuelve el siguiente movimiento dependiendo del tipo de lista
inline Movimiento ListaMovimientos::GetSiguiente(int ply) {
    switch(tipo) {
        case PERFT:
        case COMPLETA:
            return GetSiguienteCompleta(ply);
            break;
        case EVASIONES:
            return GetSiguienteEvasion(ply);
            break;
        case CAPTURAS:
            return GetSiguienteCaptura(ply);
            break;
        case COMPLETA_RAIZ:
            return GetSiguienteRaiz(ply);
            break;
        default: {
            Movimiento m;
            m.SetNulo();
            return m;
        }
    }
}

// Devuelve el puntaje del ltimo movimiento devuelto
inline int ListaMovimientos::GetPuntajeUltimoMov(void) const {
    return puntajeActual;
}

// Devuelve la cantidad de movimientos en la lista
inline int ListaMovimientos::Cantidad(void) const {
    return cantidad;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ AGREGADO DE MOVIMIENTOS
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Establece el arbol que contiene a la lista
inline void ListaMovimientos::SetArbol(const Arbol *a) {
    arbol = a;
}

// Establece el tablero al cual corresponde la lista
inline void ListaMovimientos::SetTablero(const Tablero *t) {
    tablero = t;
}

// Agrega un movimiento comn de movimiento de pieza o pen
inline void ListaMovimientos::AgregarSimple(Casilla origen, Casilla destino, Pieza pieza) {
    lista[cantidad++].SetSimple(origen, destino, pieza);
}

// Agrega una captura comn
inline void ListaMovimientos::AgregarCaptura(Casilla origen, Casilla destino, Pieza pieza, Pieza captura) {
    lista[cantidad++].SetCaptura(origen, destino, pieza, captura);
}

// Agrega una promocin de pen blanca
inline void ListaMovimientos::AgregarPromocionBlanca(Casilla origen, Casilla destino, Pieza promocion, Pieza captura) {
    lista[cantidad++].SetPromocionB(origen, destino, captura, promocion);
}

// Agrega una promocin de pen negra
inline void ListaMovimientos::AgregarPromocionNegra(Casilla origen, Casilla destino, Pieza promocion, Pieza captura) {
    lista[cantidad++].SetPromocionN(origen, destino, captura, promocion);
}

// Agrega una captura de pen al paso hecho por las blancas
inline void ListaMovimientos::AgregarAlPasoBlanco(Casilla origen, Casilla destino) {
    lista[cantidad++].SetAlPasoBlanca(origen, destino);
}

// Agrega una captura de pen al paso hecho por las negras
inline void ListaMovimientos::AgregarAlPasoNegro(Casilla origen, Casilla destino) {
    lista[cantidad++].SetAlPasoNegra(origen, destino);        
}

// Agrega un enroque corto blanco
inline void ListaMovimientos::AgregarECB(void) {
    lista[cantidad++].SetECB();
}

// Agrega un enroque corto negro
inline void ListaMovimientos::AgregarECN(void) {
    lista[cantidad++].SetECN();
}

// Agrega un enroque largo blanco
inline void ListaMovimientos::AgregarELB(void) {
    lista[cantidad++].SetELB();
}

// Agrega un enroque largo negro
inline void ListaMovimientos::AgregarELN(void) {
    lista[cantidad++].SetELN();
}
