//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Motor.cpp
//+
//+ El motor de ajedrez
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"
#include "Motor.h"
#include "Busqueda.h"
#include "Tablero.h"
#include "Evaluador.h"
#include "TablaHash.h"
#include "Config.h"
#include "TableroDebug.h"

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class Motor
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Constructor
Motor::Motor(void) {
#if defined(DEBUG_MOSTRAR)
    cout << DIVISOR << endl;
    cout << "HeavyChess - Programa de Ajedrez En Continuo Desarrollo" << endl;
    cout << "Version: " << VERSION << endl;
    cout << "Version De Desarrollo Al: " << __DATE__ << endl;
    cout << DIVISOR << endl;
#endif

    // Inicializacion
    Config::EstablecerConfiguracionInicial();
    Bitboards::InicializarBitBoards();
    Hashkeys::IniciarHashKeys();

    tablero = new Tablero();
    tableroDebug = new TableroDebug(*tablero, *this);
    tablero->SetTableroDebug(tableroDebug);

    eval = new Evaluador(*this);
    tablaHash = new TablaHash(*this);
    busqueda = new Busqueda(*this);
}

// Destructor
Motor::~Motor(void) {
    // Liberamos
    delete busqueda;
    busqueda = NULL;
    delete tablaHash;
    tablaHash = NULL;
    delete tablero;
    tablero = NULL;
    delete eval;
    eval = NULL;
    delete tableroDebug;
    tableroDebug = NULL;
}

// Cierra el motor automaticamente
void Motor::Cerrar(void) {
    // Salir
    exit(0);    
}
