//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Motor.h
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#pragma once

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ DECLARACION PREVIA DE CLASES
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

class Tablero;
class Busqueda;
class Consola;
class Evaluador;
class TablaHash;
class TableroDebug;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class Motor
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

class Motor {
public:
    // Constructor
    Motor(void);

    // Destructor
    ~Motor(void);

    // Cierra el motor automaticamente
    void Cerrar(void);

    // Asigna una consola con el motor
    void AsignarConsola(Consola *con) {
        assert(con != NULL);
        consola = con;
    }

    // Devuelve el buscador
    inline Busqueda& GetBusqueda(void) {
        assert(busqueda != NULL);
        return *busqueda;
    }

    // Devuelve el tablero
    inline Tablero& GetTablero(void) {
        assert(tablero != NULL);
        return *tablero;
    }

    // Devuelve el analizador del tablero
    inline TableroDebug& GetTableroDebug(void) {
        assert(tableroDebug != NULL);
        return *tableroDebug;
    }

    // Devuelve la consola
    inline Consola& GetConsola(void) {
        assert(consola != NULL);
        return *consola;
    }

    // Devuelve la evaluacin
    inline Evaluador& GetEval(void) {
        assert(eval != NULL);
        return *eval;
    }

    // Devuelve la tabla hash
    inline TablaHash& GetTablaHash(void) {
        assert(tablaHash != NULL);
        return *tablaHash;
    }

private:
    Busqueda *busqueda;
    Tablero *tablero;
    TableroDebug *tableroDebug;
    Consola *consola;
    Evaluador *eval;
    TablaHash *tablaHash;
};
