//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Movimiento.h
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#pragma once

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class Movimiento
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class Movimiento {
public:

    // Para obtener informacin del movimiento
    Casilla Origen(void) const;
    Casilla Destino(void) const;
    Pieza GetPieza(void) const;
    Pieza Captura(void) const;
    Pieza Promocion(void) const;
    bool EsAP(void) const;
    bool EsEnroque(void) const;
    bool EsECB(void) const;
    bool EsECN(void) const;
    bool EsELB(void) const;
    bool EsELN(void) const;
    bool EsNulo(void) const;

    // Para generar el movimiento
    void SetSimple(Casilla origen, Casilla destino, Pieza pieza);
    void SetCaptura(Casilla origen, Casilla destino, Pieza pieza, Pieza captura);
    void SetPromocionB(Casilla origen, Casilla destino, Pieza captura, Pieza promocion);
    void SetPromocionN(Casilla origen, Casilla destino, Pieza captura, Pieza promocion);
    void SetAlPasoBlanca(Casilla origen, Casilla destino);
    void SetAlPasoNegra(Casilla origen, Casilla destino);
    void SetECB(void);
    void SetECN(void);
    void SetELB(void);
    void SetELN(void);
    void SetNulo(void);

    // Para mostrar el movimiento
    void Mostrar(void) const;

    // Operadores
    bool operator==(Movimiento m) const;
    bool operator!=(Movimiento m) const;
    Movimiento& operator=(Movimiento m);

private:
    int dato;       // La representacin interna del movimiento en un entero
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Representacion de un movimiento en un integer
//+ 
//+  3130292827262524232221201918171615141312111009080706050403020100
//+ +----------------------------------------------------------------+
//+ +      AAEEEEEEEEPPPPPPPPCCCCCCCCPPPPPPPPDDDDDDDDDDDDOOOOOOOOOOOO+
//+ +----------------------------------------------------------------+
//+
//+ O = Origen (6 bits)
//+ D = Destino (6 bits) Offset = 6
//+ P = Pieza (4 bits) Offset = 12
//+ C = Captura (4 bits) Offset = 16
//+ P = Promocion (4 bits) Offset = 20
//+ E = Enroque (4 bits) Offset = 24 (ver debajo)
//+ A = AlPaso (1 bit) Offset = 28
//+
//+ Enroques = ABCD (4 bits), un bit activado indica que el movimiento en un
//                            enroque de ese tipo
//+ A = ELN
//+ B = ELB
//+ C = ECN
//+ D = ECB
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Macros
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Para los tipos de enroque
#define MOVE_ECB    1
#define MOVE_ECN    2
#define MOVE_ELB    4
#define MOVE_ELN    8

// Para extraer la informacin del entero
#define MOVE_ORIGEN(x)       static_cast<Casilla>(x & 63)
#define MOVE_DESTINO(x)      static_cast<Casilla>((x >> 6) & 63)
#define MOVE_PIEZA(x)        static_cast<Pieza>((x >> 12) & 15)
#define MOVE_CAPTURA(x)      static_cast<Pieza>((x >> 16) & 15)
#define MOVE_PROMOCION(x)    static_cast<Pieza>((x >> 20) & 15)
#define MOVE_ENROQUE(x)      ((x >> 24) & 15)
#define MOVE_AP(x)           ((x >> 28) & 1)

// Para guardar la informacin en el entero
#define SET_MOVE_ORIG(x)     (x & 63)
#define SET_MOVE_DEST(x)     ((x & 63) << 6)
#define SET_MOVE_PIEZA(x)    ((x & 15) << 12)
#define SET_MOVE_CAPT(x)     ((x & 15) << 16)
#define SET_MOVE_PROMO(x)    ((x & 15) << 20)
#define SET_MOVE_ECB         ((MOVE_ECB & 15) << 24)
#define SET_MOVE_ECN         ((MOVE_ECN & 15) << 24)
#define SET_MOVE_ELB         ((MOVE_ELB & 15) << 24)
#define SET_MOVE_ELN         ((MOVE_ELN & 15) << 24)
#define SET_MOVE_AP          (1 << 28)

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Para obtener informacin del movimiento
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

inline Casilla Movimiento::Origen(void) const { return MOVE_ORIGEN(dato); }
inline Casilla Movimiento::Destino(void) const { return MOVE_DESTINO(dato); }
inline Pieza Movimiento::GetPieza(void) const { return MOVE_PIEZA(dato); }
inline Pieza Movimiento::Captura(void) const { return MOVE_CAPTURA(dato); }
inline Pieza Movimiento::Promocion(void) const { return MOVE_PROMOCION(dato); }
inline bool Movimiento::EsAP(void) const { return (MOVE_AP(dato) != 0); }    
inline bool Movimiento::EsEnroque(void) const { return (MOVE_ENROQUE(dato) != 0); }
inline bool Movimiento::EsECB(void) const { return (MOVE_ENROQUE(dato) == MOVE_ECB); }
inline bool Movimiento::EsECN(void) const { return (MOVE_ENROQUE(dato) == MOVE_ECN); }
inline bool Movimiento::EsELB(void) const { return (MOVE_ENROQUE(dato) == MOVE_ELB); }
inline bool Movimiento::EsELN(void) const { return (MOVE_ENROQUE(dato) == MOVE_ELN); }
inline bool Movimiento::EsNulo(void) const { return dato == 0; }

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Para generar el movimiento
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Genera un movimiento simple de movimiento de pieza o pen
inline void Movimiento::SetSimple(Casilla o, Casilla d, Pieza p) {
    assert(CasillaValida(o));
    assert(CasillaValida(d));
    assert(PiezaValida(p));
    dato = SET_MOVE_PIEZA(p) | SET_MOVE_DEST(d) | SET_MOVE_ORIG(o);
}

// Genera una captura de pieza o pen
inline void Movimiento::SetCaptura(Casilla o, Casilla d, Pieza p, Pieza c) {
    assert(CasillaValida(o));
    assert(CasillaValida(d));
    assert(PiezaValida(p));
    assert(PiezaValida(c));
    dato = SET_MOVE_CAPT(c) | SET_MOVE_PIEZA(p) | SET_MOVE_DEST(d) | SET_MOVE_ORIG(o);
}

// Genera una promocin blanca
inline void Movimiento::SetPromocionB(Casilla o, Casilla d, Pieza c, Pieza p) {
    assert(CasillaValida(o));
    assert(CasillaValida(d));
    assert(PiezaValida(c));
    assert(PiezaValida(p));
    dato = SET_MOVE_PROMO(p) | SET_MOVE_CAPT(c) | SET_MOVE_PIEZA(PeonB) | SET_MOVE_DEST(d) | SET_MOVE_ORIG(o);
}

// Genera una promocin blanca
inline void Movimiento::SetPromocionN(Casilla o, Casilla d, Pieza c, Pieza p) {
    assert(CasillaValida(o));
    assert(CasillaValida(d));
    assert(PiezaValida(c));
    assert(PiezaValida(p));
    dato = SET_MOVE_PROMO(p) | SET_MOVE_CAPT(c) | SET_MOVE_PIEZA(PeonN) | SET_MOVE_DEST(d) | SET_MOVE_ORIG(o);
}

// Genera una captura al paso blanca
inline void Movimiento::SetAlPasoBlanca(Casilla o, Casilla d) {
    assert(CasillaValida(o));
    assert(CasillaValida(d));
    dato = SET_MOVE_AP | SET_MOVE_CAPT(PeonN) | SET_MOVE_PIEZA(PeonB) | SET_MOVE_DEST(d) | SET_MOVE_ORIG(o);
}

// Genera una captura al paso negra
inline void Movimiento::SetAlPasoNegra(Casilla o, Casilla d) {
    assert(CasillaValida(o));
    assert(CasillaValida(d));
    dato = SET_MOVE_AP | SET_MOVE_CAPT(PeonB) | SET_MOVE_PIEZA(PeonN) | SET_MOVE_DEST(d) | SET_MOVE_ORIG(o);
}

// Genera un enroque corto blanco
inline void Movimiento::SetECB(void) {
    dato = SET_MOVE_ECB | SET_MOVE_PIEZA(ReyB) | SET_MOVE_DEST(G1) | SET_MOVE_ORIG(E1);
}

// Genera un enroque corto negro
inline void Movimiento::SetECN(void) {
    dato = SET_MOVE_ECN | SET_MOVE_PIEZA(ReyN) | SET_MOVE_DEST(G8) | SET_MOVE_ORIG(E8);
}

// Genera un enroque largo blanco
inline void Movimiento::SetELB(void) {
    dato = SET_MOVE_ELB | SET_MOVE_PIEZA(ReyB) | SET_MOVE_DEST(C1) | SET_MOVE_ORIG(E1);
}

// Genera un enroque largo negro
inline void Movimiento::SetELN(void) {
    dato = SET_MOVE_ELN | SET_MOVE_PIEZA(ReyN) | SET_MOVE_DEST(C8) | SET_MOVE_ORIG(E8);
}

// Genera un movimiento nulo, sin info
inline void Movimiento::SetNulo(void) {
    dato = 0;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Para mostrar el movimiento
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

inline void Movimiento::Mostrar(void) const {
    cout << Notador::ObtenerNotacion(*this);
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Operadores
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

inline bool Movimiento::operator==(Movimiento m) const { return (dato == m.dato); }
inline bool Movimiento::operator!=(Movimiento m) const { return (dato != m.dato); }
inline Movimiento& Movimiento::operator=(Movimiento m) { dato = m.dato; return *this; }

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Para mostrar en pantalla un movimiento
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

inline ostream& operator<<(ostream& out, Movimiento m) {
    m.Mostrar();
    return out;
}
