//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Notador.cpp
//+
//+ Para representar la notacin en coordenadas de un movimiento
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"
#include "Notador.h"
#include "ListaMovimientos.h"
#include "Tablero.h"

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ CONSTANTES
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

const string Notador::figurinesTablero = ".PpNnBbRrQqKk";
const string Notador::figurinesPromocion = ".ppnnbbrrqqkk";

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class Notador
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Devuelve las coordenadas en un string (a4, e2, etc) de una casilla en el tablero
string Notador::ObtenerCoordenadas(Casilla casilla) {
    assert(CasillaValida(casilla));
    string ret = "";
    ret += (char)(Columna(casilla) + 'a');
    ret += (char)(Fila(casilla) + '1');
    return ret;
}

// Devuelve el figurn de una pieza para el tablero por pantalla
string Notador::ObtenerFigurin(Pieza pieza) {
    assert(PiezaValida(pieza));
    string ret = "";
    ret += figurinesTablero[pieza];
    return ret;
}

// Devuelve la notacin del movimiento en coordenadas
string Notador::ObtenerNotacion(Movimiento m) {
    if(m.EsNulo())
        return "xxxx";

    string ret = ObtenerCoordenadas(m.Origen()) + ObtenerCoordenadas(m.Destino());
    if(m.Promocion() != Vacio)
        ret += figurinesPromocion[m.Promocion()];
    return ret;
}

// Devuelve el movimiento legal en el tablero correspondiente a la notacin de coordenadas
Movimiento Notador::ObtenerMovimiento(string notacion, Tablero &tablero) {
    ListaMovimientos listaMovs(&tablero);
    Movimiento m;

    listaMovs.SetTipoLista(PERFT);
    while((m = listaMovs.GetSiguiente()).EsNulo() == false) {
        string ret = ObtenerNotacion(m);
        if(ret == notacion)
            return m;
    }

    m.SetNulo();
    return m;
}

// Devuelve la casilla del tablero correspondiente a unas coordenadas en un string
Casilla Notador::ObtenerCasilla(std::string coordenadas) {
    assert(coordenadas.length() == 2);
    transform(coordenadas.begin(),coordenadas.end(),coordenadas.begin(),::tolower);
    return (Coord(coordenadas[1]-'1',coordenadas[0]-'a'));
}
