//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ OpcionUCI.cpp
//+
//+ Una opcion en el protocolo UCI
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"
#include "OpcionUCI.h"
#include "ConsolaUCI.h"

namespace UCI {

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class OpcionUCI
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Constructor
OpcionUCI::OpcionUCI(string nombre, ConsolaUCI *con, void(ConsolaUCI::*func)(string &))
: nombreOpcion(nombre), consola(con), funcionEjecucion(func) { }

// Destructor
OpcionUCI::~OpcionUCI(void) { }

// Devuelve el nombre de la opcin
string OpcionUCI::GetNombre(void) const {
    return nombreOpcion;
}

// Muestra la opcin por pantalla
void OpcionUCI::Mostrar(void) const {
    (*consola).Salida() << "option name " << nombreOpcion << " type ";
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class OpcionUCISpin
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Constructor
OpcionUCISpin::OpcionUCISpin(string nombre, ConsolaUCI* con, void(ConsolaUCI::*func)(string &), int valorMax, int valorMin, int valor)
: OpcionUCI(nombre, con, func), valorMaximo(valorMax), valorMinimo(valorMin), valorOpcion(valor) { }

// Destructor
OpcionUCISpin::~OpcionUCISpin(void) { }

// Establece el valor
bool OpcionUCISpin::SetValor(int valor) {
    if(valor >= valorMinimo && valor <= valorMaximo) {
        valorOpcion = valor;
        return true;
    }
    return false;
}

// Devuelve el valor mximo
int OpcionUCISpin::GetMaximo(void) const {
    return valorMaximo;
}

// Devuelve el valor mnimo
int OpcionUCISpin::GetMinimo(void) const {
    return valorMinimo;
}

// Muestra la opcin por pantalla
void OpcionUCISpin::Mostrar(void) const {
    OpcionUCI::Mostrar();
    (*consola).Salida() << "spin" << " default " << valorOpcion << " min " << valorMinimo << " max " << valorMaximo << endl;
}

// Establece el nuevo valor de la opcin
void OpcionUCISpin::SetValor(string valor) {
    if(SetValor(StringAEntero(valor)))
        (consola->*(funcionEjecucion))(valor);
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class OpcionUCICheck
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Constructor
OpcionUCICheck::OpcionUCICheck(string nombre, ConsolaUCI* con, void(ConsolaUCI::*func)(string &), bool activadoDefecto)
: OpcionUCI(nombre, con, func), valorOpcion(activadoDefecto) { }

// Destructor
OpcionUCICheck::~OpcionUCICheck(void) { }

// Habilita la opcin
void OpcionUCICheck::Habilitar(void) {
    valorOpcion = true;
}

// Deshabilita la opcin
void OpcionUCICheck::Deshabilitar(void) {
    valorOpcion = false;
}

// Muestra la opcin por pantalla
void OpcionUCICheck::Mostrar(void) const {
    OpcionUCI::Mostrar();
    (*consola).Salida() << "check" << " default " << ValorVerdad(valorOpcion) << endl;
}

// Establece el nuevo valor de la opcin
void OpcionUCICheck::SetValor(string valor) {
    if(valor == "true")
        Habilitar();
    else
        Deshabilitar();
    (consola->*(funcionEjecucion))(valor);
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class OpcionUCIString
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Constructor
OpcionUCIString::OpcionUCIString(string nombre, ConsolaUCI* con, void(ConsolaUCI::*func)(string &), string valor)
: OpcionUCI(nombre, con, func), valorOpcion(valor) { }

// Destructor
OpcionUCIString::~OpcionUCIString(void) { }

// Muestra la opcin por pantalla
void OpcionUCIString::Mostrar(void) const {
    OpcionUCI::Mostrar();
    (*consola).Salida() << "string" << " default " << valorOpcion << endl;
}

// Establece el nuevo valor de la opcin
void OpcionUCIString::SetValor(string valor) {
    valorOpcion = valor;
    (consola->*(funcionEjecucion))(valor);
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class OpcionUCIButton
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Constructor
OpcionUCIButton::OpcionUCIButton(string nombre, ConsolaUCI* con, void(ConsolaUCI::*func)(string &))
: OpcionUCI(nombre, con, func) { }

// Destructor
OpcionUCIButton::~OpcionUCIButton(void) { }

// Muestra la opcin por pantalla
void OpcionUCIButton::Mostrar(void) const {
    OpcionUCI::Mostrar();
    (*consola).Salida() << "button" << endl;
}

// Establece el nuevo valor de la opcin
void OpcionUCIButton::SetValor(string valor) {
    (consola->*(funcionEjecucion))(valor);
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class OpcionUCICombo
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Constructor
OpcionUCICombo::OpcionUCICombo(string nombre, ConsolaUCI* con, void(ConsolaUCI::*func)(string &), vector<string> listaPosibles, string valor)
: OpcionUCI(nombre, con, func), valorOpcion(valor), posiblesValores(listaPosibles) { }

// Destructor
OpcionUCICombo::~OpcionUCICombo(void) { }

// Muestra la opcin por pantalla
void OpcionUCICombo::Mostrar(void) const {
    OpcionUCI::Mostrar();
    (*consola).Salida() << "combo" << " default" << valorOpcion;
    for(vector<string>::const_iterator it = posiblesValores.begin(); it != posiblesValores.end(); it++)
        (*consola).Salida() << "var " << *it;
    (*consola).Salida() << endl;
}

// Establece el nuevo valor de la opcin
void OpcionUCICombo::SetValor(string valor) {
    for(vector<string>::const_iterator it = posiblesValores.begin(); it != posiblesValores.end(); it++) {
        if(valor == *it) {
            valorOpcion = valor;
            (consola->*(funcionEjecucion))(valor);
            break;
        }
    }
}

}; // namespace UCI
