//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ OpcionUCI.h
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#pragma once

namespace UCI {

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ DECLARACION PREVIA DE CLASES
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

class ConsolaUCI;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class OpcionUCI
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

class OpcionUCI {
public:
    // Constructor
    OpcionUCI(string nombre, ConsolaUCI* con, void(ConsolaUCI::*func)(string &));

    // Destructor
    ~OpcionUCI(void);

    // Devuelve el nombre de la opcin
    string GetNombre(void) const;

    // Muestra la opcin por pantalla
    virtual void Mostrar(void) const;

    // Establece el nuevo valor de la opcin
    virtual void SetValor(string valor) = 0;

protected:
    string nombreOpcion;
    ConsolaUCI* consola;
    void (ConsolaUCI::*funcionEjecucion)(string&);
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class OpcionUCISpin
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

class OpcionUCISpin : public OpcionUCI {
public:
    // Constructor
    OpcionUCISpin(string nombre, ConsolaUCI* con, void(ConsolaUCI::*func)(string &), int valorMax, int valorMin, int valor);

    // Destructor
    ~OpcionUCISpin(void);

    // Muestra la opcin por pantalla
    virtual void Mostrar(void) const;

    // Establece el nuevo valor de la opcin
    virtual void SetValor(string valor);

private:
    // Devuelve el valor mximo
    int GetMaximo(void) const;

    // Devuelve el valor mnimo
    int GetMinimo(void) const;

    // Establece el valor
    bool SetValor(int valor);

    int valorMaximo, valorMinimo, valorOpcion;
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class OpcionUCICheck
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

class OpcionUCICheck : public OpcionUCI {
public:
    // Constructor
    OpcionUCICheck(string nombre, ConsolaUCI* con, void(ConsolaUCI::*func)(string &), bool activadoDefecto);

    // Destructor
    ~OpcionUCICheck(void);

    // Muestra la opcin por pantalla
    virtual void Mostrar(void) const;

    // Establece el nuevo valor de la opcin
    virtual void SetValor(string valor);

private:
    // Habilita la opcin
    void Habilitar(void);

    // Deshabilita la opcin
    void Deshabilitar(void);

    bool valorOpcion;
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class OpcionUCIString
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

class OpcionUCIString : public OpcionUCI {
public:
    // Constructor
    OpcionUCIString(string nombre, ConsolaUCI* con, void(ConsolaUCI::*func)(string &), string valor);

    // Destructor
    ~OpcionUCIString(void);

    // Muestra la opcin por pantalla
    virtual void Mostrar(void) const;

    // Establece el nuevo valor de la opcin
    virtual void SetValor(string valor);

private:
    string valorOpcion;
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class OpcionUCIButton
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

class OpcionUCIButton : public OpcionUCI {
public:
    // Constructor
    OpcionUCIButton(string nombre, ConsolaUCI* con, void(ConsolaUCI::*func)(string &));

    // Destructor
    ~OpcionUCIButton(void);

    // Muestra la opcin por pantalla
    virtual void Mostrar(void) const;

    // Establece el nuevo valor de la opcin
    virtual void SetValor(string valor);
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class OpcionUCICombo
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

class OpcionUCICombo : public OpcionUCI {
public:
    // Constructor
    OpcionUCICombo(string nombre, ConsolaUCI* con, void(ConsolaUCI::*func)(string &), vector<string> listaPosibles, string valor);

    // Destructor
    ~OpcionUCICombo(void);

    // Muestra la opcin por pantalla
    virtual void Mostrar(void) const;

    // Establece el nuevo valor de la opcin
    virtual void SetValor(string valor);

private:
    string valorOpcion;
    vector<string> posiblesValores;
};

}; // namespace UCI
