//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ SalidaUCI.cpp
//+
//+ Soporte de la salida en el protocolo UCI
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"
#include "SalidaUCI.h"
#include "Busqueda.h"

namespace UCI {

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class SalidaUCI
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Informa que una nueva profundidad se ha alcanzado
void SalidaUCI::NuevaProfundidadBusqueda(int prof) {
    *this << "info depth " << prof << endl;
}

// Informa que se est estudiando un nuevo movimiento en raz
void SalidaUCI::NuevoMovimientoRaiz(Movimiento m, int numMovimiento, EnteroU64 nodos) {
    *this << "info currmovenumber " << numMovimiento << " currmove " << m << " nodes " << nodos << endl;
}

// Informa que se ha encontrado una nueva variante principal
void SalidaUCI::NuevaVariantePrincipal(int prof, int profSelectiva, TipoTiempo tiempoTranscurrido, EnteroU64 nodos,
                                       double utilizacionHash, int puntaje, bool esMate, string pv) {
    *this << "info depth " << prof << " seldepth " << profSelectiva << " time " << tiempoTranscurrido
          << " nodes " << nodos << " hashfull " << static_cast<int>(utilizacionHash*10);
    if(tiempoTranscurrido)
          *this << " nps " << (nodos*1000)/(tiempoTranscurrido+1);
    if(esMate)
        *this << " score mate " << puntaje;
    else
        *this << " score cp " << puntaje;
    *this << " pv " << pv << endl;
}

// Informa el mejor movimiento devuelto
void SalidaUCI::DevolverMejorMovimiento(Movimiento mov, Movimiento ponder) {
    *this << "bestmove " << mov;
    if(!ponder.EsNulo())
        *this << " ponder " << ponder;
    *this << endl;
}

}; // namespace UCI
