//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ TablaHash.cpp
//+
//+ Para la de las tablas hash de transposiciones
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"
#include "Busqueda.h"
#include "TablaHash.h"
#include "Estadisticas.h"

using namespace Estadisticas;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class TablaHash
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Constructor
TablaHash::TablaHash(Motor &m) : motor(m) {
    tabla = NULL;
    Alojar(32); // Asigna un tamao inicial de 32 megas
}

// Destructor
TablaHash::~TablaHash(void) {
    Liberar();
}

// Aloja el espacio para la tabla
void TablaHash::Alojar(int numeroMegas) {
    // Liberamos antes la tabla anterior
    Liberar();
    
    // Se crean las tablas
    numeroEntradas = (numeroMegas * 1024 * 1024) / sizeof(EntradaHashDoble);
    if((tabla = new EntradaHashDoble[numeroEntradas]) == NULL || numeroEntradas == 0) {
        cout << "Problema Creando Tabla!" << endl;
        motor.Cerrar();
    }

#if defined(DEBUG_MOSTRAR)
    cout << "Tabla Hash: " << numeroEntradas << " entradas * " << sizeof(EntradaHashDoble) << " bytes = "
         << numeroEntradas * sizeof(EntradaHashDoble) << " bytes" << endl;
#endif

    // Se limpia el contenido de las tablas
    Limpiar();
}

// Limpia el contenido de la tabla
void TablaHash::Limpiar(void) {
    memset(tabla, 0, numeroEntradas * sizeof(EntradaHashDoble));
    memset(entradasUtilizadasDP, 0, sizeof(entradasUtilizadasDP));
    entradasUtilizadasAR = 0;
    edadDP = 1;
}

// Libera la memoria utilizada por la tabla
void TablaHash::Liberar(void) {
    delete[] tabla;
    tabla = NULL;
}

// Actuliza la edad de la tabla
void TablaHash::ActualizarEdad(void) {
    if(++edadDP > MAX_EDAD_TABLA_DP)
        edadDP = 1;
}

// Devuelve la utilizacion de la tabla en porcentaje
double TablaHash::GetUtilizacion(void) const {
    double ret = (entradasUtilizadasDP[edadDP-1]*100) + entradasUtilizadasAR;
    ret /= numeroEntradas;
    return ret;
}

// Devuelve el lugar en la tabla que le correspondera a una posicin
inline EntradaHashDoble* TablaHash::LugarEnTabla(const Hashkey &hashkey) {
    return &tabla[(unsigned int)hashkey % numeroEntradas];
}

// Guarda una entrada en la tabla
void TablaHash::Guardar(const Hashkey &hashkey, int ply, int puntaje, HashTipoLimite tipoLimite,
                        Movimiento movimiento, int profundidad, bool amenaza) {
    // Creamos un puntero al lugar en la tabla
    EntradaHashDoble *ptr = LugarEnTabla(hashkey);
    EntradaHash *temp;
    
    // Corregimos los puntajes de mate
    if(puntaje > Busqueda::MatePositivo)
        puntaje += ply;
    else if(puntaje < Busqueda::MateNegativo)
        puntaje -= ply;


    // Primero probamos con la entrada DP (Depth-Preferred)
    temp = ptr->GetEntradaDP();
    if(temp->GetProfundidad() <= profundidad || temp->GetEdad() != edadDP) {
		// Para conocer la utilizacin de la tabla hash
        if(temp->GetEdad() != edadDP) {
			if(!temp->GetEdad())
				entradasUtilizadasDP[edadDP-1]++; 
            else {
                entradasUtilizadasDP[temp->GetEdad()-1]--;
                entradasUtilizadasDP[edadDP-1]++;
            }
		}

        hashStoresDP++;
        temp->SetMovimiento(movimiento);
        temp->SetHashkey(hashkey);
        temp->SetFlags(profundidad, puntaje, tipoLimite, amenaza, edadDP);
    }
    else {
        // Si no podemos, lo hacemos en la entrada AR (Always Replace)
        temp = ptr->GetEntradaAR();

        // Para conocer la utilizacin de la tabla hash
        if(!temp->GetHashkey())
            entradasUtilizadasAR++;

        hashStoresAR++;
        temp->SetMovimiento(movimiento);
        temp->SetHashkey(hashkey);
        temp->SetFlags(profundidad, puntaje, tipoLimite, amenaza);
    }
}

// Busca en la tabla una posicin y devuelve true si la entrada sirve
HashTipoLimite TablaHash::Leer(const Hashkey &hashkey, int profundidad) {
    // Creamos un puntero al lugar en la tabla
    EntradaHashDoble *ptr = LugarEnTabla(hashkey);
    EntradaHash *temp;

    hashProbes++;    

    // Primero probamos con la entrada DP (Depth-Preferred)
    temp = ptr->GetEntradaDP();
    if(temp->GetHashkey() == hashkey && temp->GetEdad() == edadDP) {
        hashHitsDP++;
        ultimaLeida = temp;
        if(temp->GetProfundidad() >= profundidad)
            return temp->GetTipoLimite();
        return HashInfoSecundaria;
    }
    else {
        // Sino, probamos con la entrada AR (Always Replace)
        temp = ptr->GetEntradaAR();

        if(temp->GetHashkey() == hashkey) {
            hashHitsAR++;
            ultimaLeida = temp;
            if(temp->GetProfundidad() >= profundidad)
                return temp->GetTipoLimite();
            return HashInfoSecundaria;
        }
    }

    // Si ninguna funcion no hay info en la hash
    ultimaLeida = NULL;
    return HashSinInfo;
}
