//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ TablaHash.h
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#pragma once

#include "Motor.h"
#include "EntradaHash.h"

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class TablaHash
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

class TablaHash {
public:
    // Constructor
    TablaHash(Motor &motor);

    // Destructor
    ~TablaHash(void);

    // Aloja el espacio para la tabla
    void Alojar(int numeroMegas);
    
    // Limpia el contenido de la tabla
    void Limpiar(void);

    // Libera la memoria utilizada por la tabla
    void Liberar(void);

    // Guarda una entrada en la tabla
    void Guardar(const Hashkey &hashkey, int ply, int puntaje, HashTipoLimite tipoLimite,
            Movimiento movimiento, int profundidad, bool amenaza);

    // Busca en la tabla una posicin y devuelve true si la entrada sirve
    HashTipoLimite Leer(const Hashkey &hashkey, int prof);

    // Actualiza la edad de la tabla
    void ActualizarEdad(void);

    // Devuelve la utilizacion de la tabla en porcentaje
    double GetUtilizacion(void) const;

    // Devuelve informacin de la ltima entrada leda
    Movimiento GetMovimiento(void) const;
    int GetPuntaje(int ply) const;
    bool GetAmenazaMate(void) const;
    int GetProfundidad(void) const;

private:
    // Devuelve el lugar en la tabla que le correspondera a una posicin
    EntradaHashDoble* LugarEnTabla(const Hashkey &hashkey);

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ CONSTANTES
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    static const int MAX_EDAD_TABLA_DP = 7;

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //+ DATOS
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    // El motor
    Motor &motor;

    // La Tabla
    EntradaHashDoble *tabla;
    unsigned int entradasUtilizadasDP[MAX_EDAD_TABLA_DP];
    unsigned int entradasUtilizadasAR;
    unsigned int numeroEntradas;
    int edadDP;

    // Acceso a la ltima entrada leda
    EntradaHash *ultimaLeida;
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ FUNCIONES INLINE
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Devuelve el movimiento en la entrada leda
inline Movimiento TablaHash::GetMovimiento(void) const {
    return ultimaLeida->GetMovimiento();
}

// Devuelve el puntaje en la entrada leda
inline int TablaHash::GetPuntaje(int ply) const {
    int puntaje = ultimaLeida->GetPuntaje();
    if(puntaje > Busqueda::MatePositivo)
        puntaje -= ply;
    else if(puntaje < Busqueda::MateNegativo)
        puntaje += ply;
    return puntaje;
}

// Devuelve la posibilidad de amenaza de mate en la entrada leda
inline bool TablaHash::GetAmenazaMate(void) const {
    return ultimaLeida->GetAmenazaMate();
}

// Devuelve la profundidad de la entrada leda
inline int TablaHash::GetProfundidad(void) const {
    return ultimaLeida->GetProfundidad();
}
