//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ TableroDebug.cpp
//+
//+ Para mostrar por pantalla informacin sobre el tablero y chequear su
//+ estructura para encontrar errores
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"
#include "TableroDebug.h"
#include "Tablero.h"
#include "Consola.h"
#include "Bitboards.h"
#include "Hashkeys.h"
#include "Estadisticas.h"

using namespace Bitboards;
using namespace Hashkeys;
using namespace Estadisticas;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ class TableroDebug
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Constructor
TableroDebug::TableroDebug(const Tablero& t, Motor &m) : tablero(t), motor(m) {}

// Acceso a la consola
inline Consola& TableroDebug::GetConsola(void) const {
    return motor.GetConsola();
}

// Muestra los bitboards
void TableroDebug::MostrarBitboards(void) const {
#if defined(DEBUG_MOSTRAR)
    GetConsola().Salida() << "Ocupadas" << endl;
    MostrarBitboard(tablero.Ocupadas());
    GetConsola().Salida() << "Rotado90" << endl;
    MostrarBitboard(tablero.rotado90);
    GetConsola().Salida() << "Rotado45" << endl;
    MostrarBitboard(tablero.rotado45);
    GetConsola().Salida() << "Rotado315" << endl;
    MostrarBitboard(tablero.rotado315);
    GetConsola().Salida() << "piezasBlancas" << endl;
    MostrarBitboard(tablero.piezasBlancas);
    GetConsola().Salida() << "piezasNegras" << endl;
    MostrarBitboard(tablero.piezasNegras);
    GetConsola().Salida() << "PeonesB" << endl;
    MostrarBitboard(tablero.PeonesB());
    GetConsola().Salida() << "PeonesN" << endl;
    MostrarBitboard(tablero.PeonesN());
    GetConsola().Salida() << "CaballosB" << endl;
    MostrarBitboard(tablero.CaballosB());
    GetConsola().Salida() << "CaballosN" << endl;
    MostrarBitboard(tablero.CaballosN());
    GetConsola().Salida() << "AlfilesB" << endl;
    MostrarBitboard(tablero.AlfilesB());
    GetConsola().Salida() << "AlfilesN" << endl;
    MostrarBitboard(tablero.AlfilesN());
    GetConsola().Salida() << "TorresB" << endl;
    MostrarBitboard(tablero.TorresB());
    GetConsola().Salida() << "TorresN" << endl;
    MostrarBitboard(tablero.TorresN());
    GetConsola().Salida() << "DamasB" << endl;
    MostrarBitboard(tablero.DamasB());
    GetConsola().Salida() << "DamasN" << endl;
    MostrarBitboard(tablero.DamasN());
    GetConsola().Salida() << "ReyesB" << endl;
    MostrarBitboard(tablero.ReyesB());
    GetConsola().Salida() << "ReyesN" << endl;
    MostrarBitboard(tablero.ReyesN());
    GetConsola().Salida() << endl;
#endif
}

// Muestra el tablero por pantalla
void TableroDebug::Mostrar(void) const {
#if defined(DEBUG_MOSTRAR)
    for(int fila = Fila8; fila >= Fila1; fila--)  {
        for(int col = ColA; col <= ColH; col++) {
            GetConsola().Salida() << Notador::ObtenerFigurin(tablero.GetPieza(fila, col));
            GetConsola().Salida() << " ";
        }
        GetConsola().Salida() << endl;
    }

    // Alguna informacin extra
    GetConsola().Salida() << endl;
    GetConsola().Salida() << "Lado Que Mueve: " << tablero.estado.GetLadoMueve() << endl;
    GetConsola().Salida() << "Casilla Al Paso: " << (tablero.estado.GetCasillaAlPaso() != Invalida ? Notador::ObtenerCoordenadas(tablero.estado.GetCasillaAlPaso()) : "Invalida") << endl;
    GetConsola().Salida() << "Derechos De Enroque: ";
    if(tablero.estado.GetInfoEnroque().BlancasPuedenEnrocarCorto()) GetConsola().Salida() << "K";
    if(tablero.estado.GetInfoEnroque().NegrasPuedenEnrocarCorto()) GetConsola().Salida() << "k";
    if(tablero.estado.GetInfoEnroque().BlancasPuedenEnrocarLargo()) GetConsola().Salida() << "Q";
    if(tablero.estado.GetInfoEnroque().NegrasPuedenEnrocarLargo()) GetConsola().Salida() << "q";
    GetConsola().Salida() << endl << "Movidas Regla 50: " <<  tablero.estado.GetCincuenta() << endl;
    GetConsola().Salida() << "Material Blancas: " << tablero.material[Blancas] << endl;
    GetConsola().Salida() << "Material Negras: " << tablero.material[Negras] << endl;
    GetConsola().Salida() << "Total Valor Piezas Blancas: " << tablero.totalValorPiezas[Blancas] << endl;
    GetConsola().Salida() << "Total Valor Piezas Negras: " << tablero.totalValorPiezas[Negras] << endl;
    GetConsola().Salida() << "Total Peones Blancos: " << tablero.totalPeones[Blancas] << endl;
    GetConsola().Salida() << "Total Peones Negros: " << tablero.totalPeones[Negras] << endl;
    GetConsola().Salida() << "Hashkey: " << hex << tablero.hashkey << dec << endl;
    GetConsola().Salida() << "En Jaque: " << ValorVerdad(tablero.EnJaque()) << endl;
    GetConsola().Salida() << "Historial: " << tablero.historial << endl;
#endif
}

// Muestra una lista de movimientos legales
void TableroDebug::MostrarMovimientosLegales(void) const {
    GetConsola().Salida() << tablero.ListaMovimientosLegales() << endl;
    GetConsola().Salida() << "Total Movimientos Legales: " << tablero.TotalMovimientosLegales() << endl;
}

// Chequea la consistencia de las estructuras internas para encontrar bugs
bool TableroDebug::ChequearConsistencia(void) const {
#if defined(CHEQUEO_CONSISTENCIA)
    int codigoError = tablero.ChequearConsistencia();

    if(!codigoError) {
        return true;
    }
    else {
        MostrarCodigoError(codigoError);        
        return false;
    }
#else
    return true;
#endif
}

// Muestra el codigo de error del chequeo de consistencia
void TableroDebug::MostrarCodigoError(int codigoError) const {
    for(int i = 1; i < codigoError; i <<= 1) {
        switch(codigoError & i) {
            case 0x01:
                GetConsola().Salida() << "Error posicion rey blanco" << endl; break;
            case 0x02:
                GetConsola().Salida() << "Error posicion rey negro" << endl; break;
            case 0x04:
                GetConsola().Salida() << "Error bitboard piezas blancas" << endl; break;
            case 0x08:
                GetConsola().Salida() << "Error bitboard piezas negras" << endl; break;
            case 0x10:
		        GetConsola().Salida() << "Error bitboard peones blancos" << endl; break;
            case 0x20:
		        GetConsola().Salida() << "Error bitboard caballos blancos" << endl; break;
            case 0x40:
		        GetConsola().Salida() << "Error bitboard alfiles blancos" << endl; break;
            case 0x80:
		        GetConsola().Salida() << "Error bitboard torres blancas" << endl; break;
            case 0x100:
		        GetConsola().Salida() << "Error bitboard damas blancas" << endl; break;
            case 0x200:
		        GetConsola().Salida() << "Error bitboard peones negros" << endl; break;
            case 0x400:
		        GetConsola().Salida() << "Error bitboard caballos negros" << endl; break;
            case 0x800:
		        GetConsola().Salida() << "Error bitboard alfiles negros" << endl; break;
            case 0x1000:
                GetConsola().Salida() << "Error bitboard torres negras" << endl; break;
            case 0x2000:
		        GetConsola().Salida() << "Error bitboard damas negras" << endl; break;
            case 0x4000:
                GetConsola().Salida() << "Error bitboard girado 90 grados" << endl; break;
            case 0x8000:
                GetConsola().Salida() << "Error bitboard girado 45 grados" << endl; break;
            case 0x10000:
                GetConsola().Salida() << "Error bitboard girado 315 grados" << endl; break;
            case 0x20000:
                GetConsola().Salida() << "Error material blancas" << endl; break;
            case 0x40000:
                GetConsola().Salida() << "Error material negras" << endl; break;
            case 0x80000:
                GetConsola().Salida() << "Error valor piezas blancas" << endl; break;
            case 0x100000:
                GetConsola().Salida() << "Error valor piezas negras" << endl; break;
            case 0x200000:
                GetConsola().Salida() << "Error total peones blancos" << endl; break;
            case 0x400000:
                GetConsola().Salida() << "Error total peones negros" << endl; break;
            case 0x800000:
                GetConsola().Salida() << "Error hashkey" << endl; break;
        }
    }

    Mostrar();
    MostrarBitboards();
    GetConsola().Salida() << "Nodos: " << Estadisticas::nodosBusqueda << endl;
}
